# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ChrEn: Cherokee-English Machine Translation data"""


import openpyxl  # noqa: requires this pandas optional dependency for reading xlsx files
import pandas as pd

import datasets


_CITATION = """\
@inproceedings{zhang2020chren,
  title={ChrEn: Cherokee-English Machine Translation for Endangered Language Revitalization},
  author={Zhang, Shiyue and Frey, Benjamin and Bansal, Mohit},
  booktitle={EMNLP2020},
  year={2020}
}
"""

_DESCRIPTION = """\
ChrEn is a Cherokee-English parallel dataset to facilitate machine translation research between Cherokee and English.
ChrEn is extremely low-resource contains 14k sentence pairs in total, split in ways that facilitate both in-domain and out-of-domain evaluation.
ChrEn also contains 5k Cherokee monolingual data to enable semi-supervised learning.
"""

_HOMEPAGE = "https://github.com/ZhangShiyue/ChrEn"

_LICENSE = ""

_URLs = {
    "monolingual_raw": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/raw/monolingual_data.xlsx",
    "parallel_raw": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/raw/parallel_data.xlsx",
    "monolingual_chr": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/chr",
    "monolingual_en5000": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/en5000",
    "monolingual_en10000": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/en10000",
    "monolingual_en20000": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/en20000",
    "monolingual_en50000": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/en50000",
    "monolingual_en100000": "https://raw.githubusercontent.com/ZhangShiyue/ChrEn/main/data/monolingual/en100000",
    "parallel_train.chr": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/train.chr",
    "parallel_train.en": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/train.en",
    "parallel_dev.chr": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/dev.chr",
    "parallel_dev.en": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/dev.en",
    "parallel_out_dev.chr": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/out_dev.chr",
    "parallel_out_dev.en": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/out_dev.en",
    "parallel_test.chr": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/test.chr",
    "parallel_test.en": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/test.en",
    "parallel_out_test.chr": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/out_test.chr",
    "parallel_out_test.en": "https://github.com/ZhangShiyue/ChrEn/raw/main/data/parallel/out_test.en",
}


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class ChrEn(datasets.GeneratorBasedBuilder):
    """ChrEn: Cherokee-English Machine Translation data."""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="monolingual_raw", version=VERSION, description="Monolingual data with metadata"),
        datasets.BuilderConfig(name="parallel_raw", version=VERSION, description="Parallel data with metadata"),
        datasets.BuilderConfig(name="monolingual", version=VERSION, description="Monolingual data text only"),
        datasets.BuilderConfig(
            name="parallel", version=VERSION, description="Parallel data text pairs only with default split"
        ),
    ]

    DEFAULT_CONFIG_NAME = (
        "parallel"  # It's not mandatory to have a default configuration. Just use one if it make sense.
    )

    def _info(self):
        if (
            self.config.name == "monolingual_raw"
        ):  # This is the name of the configuration selected in BUILDER_CONFIGS above
            features = datasets.Features(
                {
                    "text_sentence": datasets.Value("string"),
                    "text_title": datasets.Value("string"),
                    "speaker": datasets.Value("string"),
                    "date": datasets.Value("int32"),
                    "type": datasets.Value("string"),
                    "dialect": datasets.Value("string"),
                }
            )
        elif (
            self.config.name == "parallel_raw"
        ):  # This is the name of the configuration selected in BUILDER_CONFIGS above
            features = datasets.Features(
                {
                    "line_number": datasets.Value("string"),  # doesn't always map to a number
                    "sentence_pair": datasets.Translation(languages=["en", "chr"]),
                    "text_title": datasets.Value("string"),
                    "speaker": datasets.Value("string"),
                    "date": datasets.Value("int32"),
                    "type": datasets.Value("string"),
                    "dialect": datasets.Value("string"),
                }
            )
        elif (
            self.config.name == "parallel"
        ):  # This is an example to show how to have different features for "first_domain" and "second_domain"
            features = datasets.Features(
                {
                    "sentence_pair": datasets.Translation(languages=["en", "chr"]),
                }
            )
        elif (
            self.config.name == "monolingual"
        ):  # This is an example to show how to have different features for "first_domain" and "second_domain"
            features = datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,  # Here we define them above because they are different between the two configurations
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download(_URLs)
        if self.config.name in [
            "monolingual_raw",
            "parallel_raw",
        ]:  # This is the name of the configuration selected in BUILDER_CONFIGS above
            return [
                datasets.SplitGenerator(
                    name="full",
                    gen_kwargs={
                        "filepaths": data_dir,
                        "split": "full",
                    },
                )
            ]
        elif self.config.name == "monolingual":
            return [
                datasets.SplitGenerator(
                    name=spl,
                    gen_kwargs={
                        "filepaths": data_dir,
                        "split": spl,
                    },
                )
                for spl in ["chr", "en5000", "en10000", "en20000", "en50000", "en100000"]
            ]
        else:
            return [
                datasets.SplitGenerator(
                    name=spl,
                    gen_kwargs={
                        "filepaths": data_dir,
                        "split": spl,
                    },
                )
                for spl in ["train", "dev", "out_dev", "test", "out_test"]
            ]

    def _generate_examples(self, filepaths, split):
        if self.config.name == "monolingual_raw":
            keys = ["text_sentence", "text_title", "speaker", "date", "type", "dialect"]
            with open(filepaths["monolingual_raw"], "rb") as f:
                monolingual = pd.read_excel(f, engine="openpyxl")
                for id_, row in enumerate(monolingual.itertuples()):
                    yield id_, dict(zip(keys, row[1:]))
        elif self.config.name == "parallel_raw":
            keys = ["line_number", "en_sent", "chr_sent", "text_title", "speaker", "date", "type", "dialect"]
            with open(filepaths["parallel_raw"], "rb") as f:
                parallel = pd.read_excel(f, engine="openpyxl")
                for id_, row in enumerate(parallel.itertuples()):
                    res = dict(zip(keys, row[1:]))
                    res["sentence_pair"] = {"en": res["en_sent"], "chr": res["chr_sent"]}
                    res["line_number"] = str(res["line_number"])
                    del res["en_sent"]
                    del res["chr_sent"]
                    yield id_, res
        elif self.config.name == "monolingual":
            f = open(filepaths[f"monolingual_{split}"], encoding="utf-8")
            for id_, line in enumerate(f):
                yield id_, {"sentence": line.strip()}
        elif self.config.name == "parallel":
            fi = open(filepaths[f"parallel_{split}.en"], encoding="utf-8")
            fo = open(filepaths[f"parallel_{split}.chr"], encoding="utf-8")
            for id_, (line_en, line_chr) in enumerate(zip(fi, fo)):
                yield id_, {"sentence_pair": {"en": line_en.strip(), "chr": line_chr.strip()}}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""HotpotQA: A Dataset for Diverse, Explainable Multi-hop Question Answering."""


import json
import textwrap

import datasets


_CITATION = """
@inproceedings{yang2018hotpotqa,
  title={{HotpotQA}: A Dataset for Diverse, Explainable Multi-hop Question Answering},
  author={Yang, Zhilin and Qi, Peng and Zhang, Saizheng and Bengio, Yoshua and Cohen, William W. and Salakhutdinov, Ruslan and Manning, Christopher D.},
  booktitle={Conference on Empirical Methods in Natural Language Processing ({EMNLP})},
  year={2018}
}
"""

_DESCRIPTION = """\
HotpotQA is a new dataset with 113k Wikipedia-based question-answer pairs with four key features:
(1) the questions require finding and reasoning over multiple supporting documents to answer;
(2) the questions are diverse and not constrained to any pre-existing knowledge bases or knowledge schemas;
(3) we provide sentence-level supporting facts required for reasoning, allowingQA systems to reason with strong supervisionand explain the predictions;
(4) we offer a new type of factoid comparison questions to testQA systems’ ability to extract relevant facts and perform necessary comparison.
"""

_URL_BASE = "http://curtis.ml.cmu.edu/datasets/hotpot/"


class HotpotQA(datasets.GeneratorBasedBuilder):
    """HotpotQA is a Dataset for Diverse, Explainable Multi-hop Question Answering."""

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="distractor",
            version=datasets.Version("1.0.0"),
            description=textwrap.dedent(
                """
                     In the distractor setting, a question-answering system reads 10 paragraphs to provide an answer to a question.
                     They must also justify these answers with supporting facts. This setting challenges the model to find the true
                     supporting facts in the presence of noise, for each example we employ bigram tf-idf (Chen et al., 2017) to retrieve
                     8 paragraphs from Wikipedia as distractors, using the question as the query. We mix them with the 2 gold paragraphs
                     (the ones used to collect the question and answer) to construct the distractor setting.
                """
            ),
        ),
        datasets.BuilderConfig(
            name="fullwiki",
            version=datasets.Version("1.0.0"),
            description=textwrap.dedent(
                """
                     In the fullwiki setting, a question-answering system must find the answer to a question in the scope of the
                     entire Wikipedia. We fully test the model’s ability to locate relevant facts as well as reasoning about them
                     by requiring it to answer the question given the first paragraphs of all Wikipedia articles without the gold
                     paragraphs specified. This full wiki setting truly tests the performance of the systems’ ability at multi-hop
                     reasoning in the wild.
                """
            ),
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "type": datasets.Value("string"),
                    "level": datasets.Value("string"),
                    "supporting_facts": datasets.features.Sequence(
                        {
                            "title": datasets.Value("string"),
                            "sent_id": datasets.Value("int32"),
                        }
                    ),
                    "context": datasets.features.Sequence(
                        {
                            "title": datasets.Value("string"),
                            "sentences": datasets.features.Sequence(datasets.Value("string")),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://hotpotqa.github.io/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        paths = {
            datasets.Split.TRAIN: _URL_BASE + "hotpot_train_v1.1.json",
            datasets.Split.VALIDATION: _URL_BASE + "hotpot_dev_" + self.config.name + "_v1.json",
        }
        if self.config.name == "fullwiki":
            paths[datasets.Split.TEST] = _URL_BASE + "hotpot_test_fullwiki_v1.json"

        files = dl_manager.download(paths)

        split_generators = []
        for split in files:
            split_generators.append(datasets.SplitGenerator(name=split, gen_kwargs={"data_file": files[split]}))

        return split_generators

    def _generate_examples(self, data_file):
        """This function returns the examples."""
        data = json.load(open(data_file))
        for idx, example in enumerate(data):

            # Test set has missing keys
            for k in ["answer", "type", "level"]:
                if k not in example.keys():
                    example[k] = None

            if "supporting_facts" not in example.keys():
                example["supporting_facts"] = []

            yield idx, {
                "id": example["_id"],
                "question": example["question"],
                "answer": example["answer"],
                "type": example["type"],
                "level": example["level"],
                "supporting_facts": [{"title": f[0], "sent_id": f[1]} for f in example["supporting_facts"]],
                "context": [{"title": f[0], "sentences": f[1]} for f in example["context"]],
            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Wikipedia knowledge source for KILT"""


import json

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{fb_kilt,
    author    = {Fabio Petroni and
                 Aleksandra Piktus and
                 Angela Fan and
                 Patrick Lewis and
                 Majid Yazdani and
                 Nicola De Cao and
                 James Thorne and
                 Yacine Jernite and
                 Vassilis Plachouras and
                 Tim Rockt\"aschel and
                 Sebastian Riedel},
    title     = {{KILT:} a {B}enchmark for {K}nowledge {I}ntensive {L}anguage {T}asks},
    journal   = {CoRR},
    archivePrefix = {arXiv},
    year      = {2020},
"""

_DESCRIPTION = """\
KILT-Wikipedia: Wikipedia pre-processed for KILT.
"""


class KILTWikipediaConfig(datasets.BuilderConfig):
    """BuilderConfig for KILTWikipedia."""

    def __init__(self, **kwargs):
        """BuilderConfig for KILTWikipedia.

            Args:
        .
              **kwargs: keyword arguments forwarded to super.
        """
        super(KILTWikipediaConfig, self).__init__(
            version=datasets.Version("1.0.0", "Wikipedia pre-processed for KILT"), **kwargs
        )


class KILTWikipedia(datasets.GeneratorBasedBuilder):
    """KILTWikipedia: Wikipedia pre-processed for KILT. Version 1.0."""

    BUILDER_CONFIGS = [
        KILTWikipediaConfig(
            name="2019-08-01",
            description="Wikipedia pre-processed for KILT from 2019/08/01 dump",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "kilt_id": datasets.Value("string"),
                    "wikipedia_id": datasets.Value("string"),
                    "wikipedia_title": datasets.Value("string"),
                    "text": datasets.features.Sequence({"paragraph": datasets.Value("string")}),
                    "anchors": datasets.features.Sequence(
                        {
                            "paragraph_id": datasets.Value("int32"),
                            "start": datasets.Value("int32"),
                            "end": datasets.Value("int32"),
                            "text": datasets.Value("string"),
                            "href": datasets.Value("string"),
                            "wikipedia_title": datasets.Value("string"),
                            "wikipedia_id": datasets.Value("string"),
                        }
                    ),
                    "categories": datasets.Value("string"),
                    "wikidata_info": datasets.Features(
                        {
                            "description": datasets.Value("string"),
                            "enwikiquote_title": datasets.Value("string"),
                            "wikidata_id": datasets.Value("string"),
                            "wikidata_label": datasets.Value("string"),
                            "wikipedia_title": datasets.Value("string"),
                            "aliases": datasets.features.Sequence({"alias": datasets.Value("string")}),
                        }
                    ),
                    "history": datasets.Features(
                        {
                            "pageid": datasets.Value("int32"),
                            "parentid": datasets.Value("int32"),
                            "revid": datasets.Value("int32"),
                            "pre_dump": datasets.Value("bool"),
                            "timestamp": datasets.Value("string"),
                            "url": datasets.Value("string"),
                        }
                    ),
                }
            ),
            # No default supervised_keys (as we have to pass both premise
            # and hypothesis as input).
            supervised_keys=None,
            homepage="https://github.com/facebookresearch/KILT",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):

        downloaded_path = dl_manager.download_and_extract(
            "http://dl.fbaipublicfiles.com/KILT/kilt_knowledgesource.json"
        )

        return [
            datasets.SplitGenerator(name="full", gen_kwargs={"filepath": downloaded_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Wikipedia articles for KILT.

        Args:
          filepath: a string

        Yields:
          dictionaries representing article data and metadata
        """
        logger.info("generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            for idx, line in enumerate(f):
                pre_article = json.loads(line.strip())
                article = dict([(k, pre_article[k]) for k in ["wikipedia_id", "wikipedia_title", "categories"]])
                # wikidata
                article["wikidata_info"] = {}
                pre_article["wikidata_info"] = pre_article.get("wikidata_info", {})
                if pre_article["wikidata_info"].get("aliases", None) is None:
                    pre_article["wikidata_info"]["aliases"] = []
                for k in ["description", "enwikiquote_title", "wikidata_id", "wikidata_label", "wikipedia_title"]:
                    val = pre_article["wikidata_info"].get(k, None)
                    article["wikidata_info"][k] = "" if val is None else val
                article["wikidata_info"]["aliases"] = {"alias": pre_article["wikidata_info"]["aliases"]}
                # history
                article["history"] = {}
                pre_article["history"] = pre_article.get("history", {})
                pre_dump = pre_article["history"].get("pre_dump", None)
                article["history"]["pre_dump"] = False if pre_dump is None else pre_dump
                for k in ["pageid", "parentid", "revid"]:
                    val = pre_article["history"].get(k, None)
                    article["history"][k] = -1 if val is None else val
                for k in ["timestamp", "url"]:
                    val = pre_article["history"].get(k, None)
                    article["history"][k] = "" if val is None else val
                # everything else
                article["kilt_id"] = pre_article["_id"]
                article["text"] = {"paragraph": pre_article["text"]}
                article["anchors"] = {}
                for k in ["paragraph_id", "start", "end", "text", "href", "wikipedia_title", "wikipedia_id"]:
                    article["anchors"][k] = [anchor.get(k, "") for anchor in pre_article["anchors"]]
                yield idx, article



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Introduction to AMTTL CWS Dataset"""

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{xing2018adaptive,
  title={Adaptive multi-task transfer learning for Chinese word segmentation in medical text},
  author={Xing, Junjie and Zhu, Kenny and Zhang, Shaodian},
  booktitle={Proceedings of the 27th International Conference on Computational Linguistics},
  pages={3619--3630},
  year={2018}
}
"""

_DESCRIPTION = """\
Chinese word segmentation (CWS) trained from open source corpus faces dramatic performance drop
when dealing with domain text, especially for a domain with lots of special terms and diverse
writing styles, such as the biomedical domain. However, building domain-specific CWS requires
extremely high annotation cost. In this paper, we propose an approach by exploiting domain-invariant
knowledge from high resource to low resource domains. Extensive experiments show that our mode
achieves consistently higher accuracy than the single-task CWS and other transfer learning
baselines, especially when there is a large disparity between source and target domains.

This dataset is the accompanied medical Chinese word segmentation (CWS) dataset.
The tags are in BIES scheme.

For more details see https://www.aclweb.org/anthology/C18-1307/
"""

_URL = "https://raw.githubusercontent.com/adapt-sjtu/AMTTL/master/medical_data/"
_TRAINING_FILE = "forum_train.txt"
_DEV_FILE = "forum_dev.txt"
_TEST_FILE = "forum_test.txt"


class AmttlConfig(datasets.BuilderConfig):
    """BuilderConfig for AMTTL"""

    def __init__(self, **kwargs):
        """BuilderConfig for AMTTL.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(AmttlConfig, self).__init__(**kwargs)


class Amttl(datasets.GeneratorBasedBuilder):
    """AMTTL Chinese Word Segmentation dataset."""

    BUILDER_CONFIGS = [
        AmttlConfig(
            name="amttl",
            version=datasets.Version("1.0.0"),
            description="AMTTL medical Chinese word segmentation dataset",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "B",
                                "I",
                                "E",
                                "S",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://www.aclweb.org/anthology/C18-1307/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls_to_download = {
            "train": f"{_URL}{_TRAINING_FILE}",
            "dev": f"{_URL}{_DEV_FILE}",
            "test": f"{_URL}{_TEST_FILE}",
        }
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            guid = 0
            tokens = []
            tags = []
            for line in f:
                line_stripped = line.strip()
                if line_stripped == "":
                    if tokens:
                        yield guid, {
                            "id": str(guid),
                            "tokens": tokens,
                            "tags": tags,
                        }
                        guid += 1
                        tokens = []
                        tags = []
                else:
                    splits = line_stripped.split("\t")
                    if len(splits) == 1:
                        splits.append("O")
                    tokens.append(splits[0])
                    tags.append(splits[1])
            # last example
            yield guid, {
                "id": str(guid),
                "tokens": tokens,
                "tags": tags,
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SNOW T15 and T23: "Japanese Simplified Corpus with Core Vocabulary" and ''Crowdsourced Corpus of Sentence Simplification with Core Vocabulary"."""


import openpyxl  # noqa: requires this pandas optional dependency for reading xlsx files
import pandas as pd

import datasets


_CITATION = """\
@inproceedings{maruyama-yamamoto-2018-simplified,
    title = "Simplified Corpus with Core Vocabulary",
    author = "Maruyama, Takumi  and
      Yamamoto, Kazuhide",
    booktitle = "Proceedings of the Eleventh International Conference on Language Resources and Evaluation ({LREC} 2018)",
    month = may,
    year = "2018",
    address = "Miyazaki, Japan",
    publisher = "European Language Resources Association (ELRA)",
    url = "https://www.aclweb.org/anthology/L18-1185",
}

@inproceedings{yamamoto-2017-simplified-japanese,
    title = "やさしい⽇本語対訳コーパスの構築",
    author = "⼭本 和英  and
      丸⼭ 拓海  and
      ⾓張 ⻯晴  and
      稲岡 夢⼈  and
      ⼩川 耀⼀朗  and
      勝⽥ 哲弘  and
      髙橋 寛治",
    booktitle = "言語処理学会第23回年次大会",
    month = 3月,
    year = "2017",
    address = "茨城, 日本",
    publisher = "言語処理学会",
    url = "https://www.anlp.jp/proceedings/annual_meeting/2017/pdf_dir/B5-1.pdf",
}

@inproceedings{katsuta-yamamoto-2018-crowdsourced,
    title = "Crowdsourced Corpus of Sentence Simplification with Core Vocabulary",
    author = "Katsuta, Akihiro  and
      Yamamoto, Kazuhide",
    booktitle = "Proceedings of the Eleventh International Conference on Language Resources and Evaluation ({LREC} 2018)",
    month = may,
    year = "2018",
    address = "Miyazaki, Japan",
    publisher = "European Language Resources Association (ELRA)",
    url = "https://www.aclweb.org/anthology/L18-1072",
}
"""

_DESCRIPTION = """\
About SNOW T15: \
The simplified corpus for the Japanese language. The corpus has 50,000 manually simplified and aligned sentences. \
This corpus contains the original sentences, simplified sentences and English translation of the original sentences. \
It can be used for automatic text simplification as well as translating simple Japanese into English and vice-versa. \
The core vocabulary is restricted to 2,000 words where it is selected by accounting for several factors such as meaning preservation, variation, simplicity and the UniDic word segmentation criterion.
For details, refer to the explanation page of Japanese simplification (http://www.jnlp.org/research/Japanese_simplification). \
The original texts are from "small_parallel_enja: 50k En/Ja Parallel Corpus for Testing SMT Methods", which is a bilingual corpus for machine translation. \
\
About SNOW T23: \
An expansion corpus of 35,000 sentences rewritten in easy Japanese (simple Japanese vocabulary) based on SNOW T15. \
The original texts are from "Tanaka Corpus" (http://www.edrdg.org/wiki/index.php/Tanaka_Corpus).
"""

_HOMEPAGE = "http://www.jnlp.org/SNOW/T15, http://www.jnlp.org/SNOW/T23"

_LICENSE = "CC BY 4.0"

# The HuggingFace dataset library don't host the datasets but only point to the original files
_URLs = {
    "snow_t15": "https://filedn.com/lit4DCIlHwxfS1gj9zcYuDJ/SNOW/T15-2020.1.7.xlsx",
    "snow_t23": "https://filedn.com/lit4DCIlHwxfS1gj9zcYuDJ/SNOW/T23-2020.1.7.xlsx",
}


class SnowSimplifiedJapaneseCorpus(datasets.GeneratorBasedBuilder):
    """SNOW T15 and T23: "Japanese Simplified Corpus with Core Vocabulary" and ''Crowdsourced Corpus of Sentence Simplification with Core Vocabulary"."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="snow_t15", version=VERSION, description="SNOW T15 dataset"),
        datasets.BuilderConfig(name="snow_t23", version=VERSION, description="SNOW T23 dataset (extension)"),
    ]

    DEFAULT_CONFIG_NAME = "snow_t15"

    def _info(self):
        if self.config.name == "snow_t15":
            features = datasets.Features(
                {
                    "ID": datasets.Value("string"),
                    "original_ja": datasets.Value("string"),
                    "simplified_ja": datasets.Value("string"),
                    "original_en": datasets.Value("string"),
                }
            )
        else:
            features = datasets.Features(
                {
                    "ID": datasets.Value("string"),
                    "original_ja": datasets.Value("string"),
                    "simplified_ja": datasets.Value("string"),
                    "original_en": datasets.Value("string"),
                    "proper_noun": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URLs[self.config.name]
        data_url = dl_manager.download(my_urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_url, "split": "train"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, "rb") as f:
            df = pd.read_excel(f, engine="openpyxl").astype("str")

            if self.config.name == "snow_t15":
                for id_, row in df.iterrows():
                    yield id_, {
                        "ID": row["ID"],
                        "original_ja": row["#日本語(原文)"],
                        "simplified_ja": row["#やさしい日本語"],
                        "original_en": row["#英語(原文)"],
                    }
            else:
                for id_, row in df.iterrows():
                    yield id_, {
                        "ID": row["ID"],
                        "original_ja": row["#日本語(原文)"],
                        "simplified_ja": row["#やさしい日本語"],
                        "original_en": row["#英語(原文)"],
                        "proper_noun": row["#固有名詞"],
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""NCSLGR: a small American Sign Language corpus annotated with non-manual features"""


import os
import re
from dataclasses import dataclass

import datasets


_DESCRIPTION = """
A small corpus of American Sign Language (ASL) video data from native signers, annotated with non-manual features.
"""

_CITATION = """\
@misc{dataset:databases2007volumes,
    title={Volumes 2--7},
    author={Databases, NCSLGR},
    year={2007},
    publisher={American Sign Language Linguistic Research Project (Distributed on CD-ROM~…}
}
"""

_URL_ANNOTATIONS = "http://asl.cs.depaul.edu/corpus/elanBUcorpus.zip"
_URL_VIDEOS = "http://asl.cs.depaul.edu/corpus/video.zip"

_HOMEPAGE = "https://www.bu.edu/asllrp/ncslgr.html"


@dataclass
class NCSLGRConfig(datasets.BuilderConfig):
    """BuilderConfig for NCSLGR."""

    videos: bool = True


class NCSLGR(datasets.GeneratorBasedBuilder):
    """NCSLGR: a small American Sign Language corpus annotated with non-manual features"""

    VERSION = datasets.Version("0.7.0")

    BUILDER_CONFIG_CLASS = NCSLGRConfig

    BUILDER_CONFIGS = [
        NCSLGRConfig(
            name="entire_dataset",
            version=datasets.Version("0.7.0"),
            description="Entire dataset containing both videos and annotations.",
            videos=True,
        ),
        NCSLGRConfig(
            name="annotations",
            version=datasets.Version("0.7.0"),
            description="Dataset including only annotations, without videos",
            videos=False,
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=datasets.Features(
                {
                    "eaf": datasets.Value("string"),  # EAF path
                    "sentences": datasets.features.Sequence(
                        {
                            "gloss": datasets.Value("string"),  # ASL Gloss
                            "text": datasets.Value("string"),  # English Text
                        }
                    ),
                    "videos": datasets.features.Sequence(datasets.Value("string")),  # Videos paths
                }
            ),
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        eaf_path = os.path.join(dl_manager.download_and_extract(_URL_ANNOTATIONS), "elanBUcorpus")
        videos_path = dl_manager.download_and_extract(_URL_VIDEOS) if self.config.videos else None

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"eaf_path": eaf_path, "videos_path": videos_path},
            )
        ]

    def _extract_sentences(self, elan_xml: str):
        def get_tier_values(name: str):
            tiers = re.findall('TIER_ID="' + name + '">([\\s\\S]*?)</TIER>', elan_xml)
            if len(tiers) == 0:
                return []
            tier = tiers[0]
            return [
                (int(s[2:]), int(e[2:]), t)
                for s, e, t in re.findall(
                    'TIME_SLOT_REF1="(.*)" TIME_SLOT_REF2="(.*)">\n.*<ANNOTATION_VALUE>(.*?)</ANNOTATION_VALUE>', tier
                )
            ]

        gloss = get_tier_values("main gloss")
        texts = get_tier_values("English translation")

        for s, e, text in texts:
            relevant_gloss = [t for (s2, e2, t) in gloss if s2 >= s and e2 <= e]
            yield {"gloss": " ".join(relevant_gloss), "text": text}

    def _generate_examples(self, eaf_path: str, videos_path: str):
        """Yields examples."""

        for i, eaf_file in enumerate(os.listdir(eaf_path)):
            eaf_file_path = os.path.join(eaf_path, eaf_file)
            videos = []
            with open(eaf_file_path, "r", encoding="utf-8") as f:
                content = f.read()

            if self.config.videos:
                videos_relative = re.findall('RELATIVE_MEDIA_URL="(.*)"', content)
                videos = [os.path.join(videos_path, v[3:]) for v in videos_relative]

            sentences = list(self._extract_sentences(content))

            yield i, {"eaf": eaf_file_path, "videos": videos, "sentences": sentences}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SGD: The Schema Guided Dialogue dataet"""


import json

import datasets


_CITATION = """\
@inproceedings{aaai/RastogiZSGK20,
  author    = {Abhinav Rastogi and
               Xiaoxue Zang and
               Srinivas Sunkara and
               Raghav Gupta and
               Pranav Khaitan},
  title     = {Towards Scalable Multi-Domain Conversational Agents: The Schema-Guided
               Dialogue Dataset},
  booktitle = {The Thirty-Fourth {AAAI} Conference on Artificial Intelligence, {AAAI}
               2020, The Thirty-Second Innovative Applications of Artificial Intelligence
               Conference, {IAAI} 2020, The Tenth {AAAI} Symposium on Educational
               Advances in Artificial Intelligence, {EAAI} 2020, New York, NY, USA,
               February 7-12, 2020},
  pages     = {8689--8696},
  publisher = {{AAAI} Press},
  year      = {2020},
  url       = {https://aaai.org/ojs/index.php/AAAI/article/view/6394}
}
"""

_DESCRIPTION = """\
The Schema-Guided Dialogue dataset (SGD) was developed for the Dialogue State Tracking task of the Eights Dialogue Systems Technology Challenge (dstc8).
The SGD dataset consists of over 18k annotated multi-domain, task-oriented conversations between a human and a virtual assistant.
These conversations involve interactions with services and APIs spanning 17 domains, ranging from banks and events to media, calendar, travel, and weather.
For most of these domains, the SGD dataset contains multiple different APIs, many of which have overlapping functionalities but different interfaces,
which reflects common real-world scenarios.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/dstc8-schema-guided-dialogue"

_LICENSE = "CC BY-SA 4.0"

_URL_LIST = [
    (
        "train_schema.json",
        "https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/train/schema.json",
    ),
    (
        "dev_schema.json",
        "https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/dev/schema.json",
    ),
    (
        "test_schema.json",
        "https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/test/schema.json",
    ),
]
_URL_LIST += [
    (
        f"train_dialogues_{i:03d}.json",
        f"https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/train/dialogues_{i:03d}.json",
    )
    for i in range(1, 128)
]
_URL_LIST += [
    (
        f"dev_dialogues_{i:03d}.json",
        f"https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/dev/dialogues_{i:03d}.json",
    )
    for i in range(1, 21)
]
_URL_LIST += [
    (
        f"test_dialogues_{i:03d}.json",
        f"https://github.com/google-research-datasets/dstc8-schema-guided-dialogue/raw/master/test/dialogues_{i:03d}.json",
    )
    for i in range(1, 35)
]

_URLs = dict(_URL_LIST)

_USER_ACTS = [
    "INFORM_INTENT",
    "NEGATE_INTENT",
    "AFFIRM_INTENT",
    "INFORM",
    "REQUEST",
    "AFFIRM",
    "NEGATE",
    "SELECT",
    "REQUEST_ALTS",
    "THANK_YOU",
    "GOODBYE",
]

_SYSTEM_ACTS = [
    "INFORM",
    "REQUEST",
    "CONFIRM",
    "OFFER",
    "NOTIFY_SUCCESS",
    "NOTIFY_FAILURE",
    "INFORM_COUNT",
    "OFFER_INTENT",
    "REQ_MORE",
    "GOODBYE",
]

_ALL_ACTS = sorted(list(set(_USER_ACTS + _SYSTEM_ACTS)))


class SchemaGuidedDstc8(datasets.GeneratorBasedBuilder):

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="dialogues", description="The dataset of annotated dialogues."),
        datasets.BuilderConfig(name="schema", description="The schemas corresponding to the API calls."),
    ]

    DEFAULT_CONFIG_NAME = "dialogues"

    def _info(self):
        if self.config.name == "schema":
            features = datasets.Features(
                {
                    "service_name": datasets.Value("string"),
                    "description": datasets.Value("string"),
                    "slots": datasets.Sequence(
                        {
                            "name": datasets.Value("string"),
                            "description": datasets.Value("string"),
                            "is_categorical": datasets.Value("bool"),
                            "possible_values": datasets.Sequence(datasets.Value("string")),
                        }
                    ),
                    "intents": datasets.Sequence(
                        {
                            "name": datasets.Value("string"),
                            "description": datasets.Value("string"),
                            "is_transactional": datasets.Value("bool"),
                            "required_slots": datasets.Sequence(datasets.Value("string")),
                            # optional_slots was originally a dictionary
                            "optional_slots": datasets.Sequence(
                                {
                                    "slot_name": datasets.Value("string"),
                                    "slot_value": datasets.Value("string"),
                                }
                            ),
                            "result_slots": datasets.Sequence(datasets.Value("string")),
                        },
                    ),
                }
            )
        else:
            features = datasets.Features(
                {
                    "dialogue_id": datasets.Value("string"),
                    "services": datasets.Sequence(datasets.Value("string")),
                    "turns": datasets.Sequence(
                        {
                            "speaker": datasets.ClassLabel(names=["USER", "SYSTEM"]),
                            "utterance": datasets.Value("string"),
                            "frames": datasets.Sequence(
                                {
                                    "service": datasets.Value("string"),
                                    "slots": datasets.Sequence(
                                        {
                                            "slot": datasets.Value("string"),
                                            "start": datasets.Value("int32"),
                                            "exclusive_end": datasets.Value("int32"),
                                        }
                                    ),
                                    # optional
                                    "state": {
                                        "active_intent": datasets.Value("string"),
                                        "requested_slots": datasets.Sequence(datasets.Value("string")),
                                        # slot_values was originally a dictionary
                                        "slot_values": datasets.Sequence(
                                            {
                                                "slot_name": datasets.Value("string"),
                                                "slot_value_list": datasets.Sequence(datasets.Value("string")),
                                            }
                                        ),
                                    },
                                    "actions": datasets.Sequence(
                                        {
                                            "act": datasets.ClassLabel(names=_ALL_ACTS),
                                            # optional
                                            "slot": datasets.Value("string"),
                                            # optional
                                            "canonical_values": datasets.Sequence(datasets.Value("string")),
                                            # optional
                                            "values": datasets.Sequence(datasets.Value("string")),
                                        }
                                    ),
                                    # optional
                                    "service_results": datasets.Sequence(
                                        # Arrow doesn't like Sequences of Sequences for default values so we need a Sequence of Features of Sequences
                                        {
                                            "service_results_list": datasets.Sequence(
                                                # originally each list item was a dictionary (optional)
                                                {
                                                    "service_slot_name": datasets.Value("string"),
                                                    "service_canonical_value": datasets.Value("string"),
                                                }
                                            )
                                        }
                                    ),
                                    # optional
                                    "service_call": {
                                        "method": datasets.Value("string"),
                                        # parameters was originally a dictionary
                                        "parameters": datasets.Sequence(
                                            {
                                                "parameter_slot_name": datasets.Value("string"),
                                                "parameter_canonical_value": datasets.Value("string"),
                                            }
                                        ),
                                    },
                                }
                            ),
                        }
                    ),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,  # Here we define them above because they are different between the two configurations
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_files = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=spl_enum,
                gen_kwargs={
                    "filepaths": data_files,
                    "split": spl,
                },
            )
            for spl, spl_enum in [
                ("train", datasets.Split.TRAIN),
                ("dev", datasets.Split.VALIDATION),
                ("test", datasets.Split.TEST),
            ]
        ]

    def _generate_examples(self, filepaths, split):
        id_ = -1
        file_list = [fpath for fname, fpath in filepaths.items() if fname.startswith(f"{split}_{self.config.name}")]
        for filepath in file_list:
            examples = json.load(open(filepath, encoding="utf-8"))
            for example in examples:
                id_ += 1
                if self.config.name == "schema":
                    example["intents"] = example.get("intents", [])
                    for intent in example["intents"]:
                        optional_slots = intent.get("optional_slots", {})
                        intent["optional_slots"] = {
                            "slot_name": list(optional_slots.keys()),
                            "slot_value": list(optional_slots.values()),
                        }
                else:
                    for turn in example["turns"]:
                        for frame in turn["frames"]:
                            # add empty state if the key is missing from the dict
                            frame["state"] = frame.get(
                                "state",
                                {
                                    "active_intent": "",
                                    "requested_slots": [],
                                    "slot_values": {},
                                },
                            )
                            # linearize the optional slot_values dictionary
                            slot_values_dict = frame["state"].get("slot_values", {})
                            frame["state"]["slot_values"] = {
                                "slot_name": list(slot_values_dict.keys()),
                                "slot_value_list": list(slot_values_dict.values()),
                            }
                            # add default values for optional fields in actions
                            for action in frame["actions"]:
                                action["slot"] = action.get("slot", "")
                                action["canonical_values"] = action.get("canonical_values", [])
                                action["values"] = action.get("values", [])
                            # add "service_results" field when necessary and linearize the dictionaries in the list otherwise
                            service_results = []
                            for result in frame.get("service_results", []):
                                service_results += [
                                    {
                                        "service_slot_name": list(result.keys()),
                                        "service_canonical_value": list(result.values()),
                                    }
                                ]
                            frame["service_results"] = {
                                "service_results_list": service_results,
                            }
                            # add "service_call" field when necessary and linearize the parameters dictionary otherwise
                            frame["service_call"] = frame.get(
                                "service_call",
                                {
                                    "method": "",
                                    "parameters": {},
                                },
                            )
                            parameters_dict = frame["service_call"].get("parameters", {})
                            frame["service_call"]["parameters"] = {
                                "parameter_slot_name": list(parameters_dict.keys()),
                                "parameter_canonical_value": list(parameters_dict.values()),
                            }
                yield id_, example



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Corpus of domain names scraped from Common Crawl and manually annotated to add word boundaries (e.g. "commoncrawl" to "common crawl")."""


import datasets


# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@inproceedings{zrs2020urlsegmentation,
  title={Semi-supervised URL Segmentation with Recurrent Neural Networks Pre-trained on Knowledge Graph Entities},
  author={Hao Zhang and Jae Ro and Richard William Sproat},
  booktitle={The 28th International Conference on Computational Linguistics (COLING 2020)},
  year={2020}
}
"""


_DESCRIPTION = """Corpus of domain names scraped from Common Crawl and manually annotated to add word boundaries (e.g. "commoncrawl" to "common crawl"). Breaking domain names such as "openresearch" into component words "open" and "research" is important for applications such as Text-to-Speech synthesis and web search. Common Crawl is an open repository of web crawl data that can be accessed and analyzed by anyone. Specifically, we scraped the plaintext (WET) extracts for domain names from URLs that contained diverse letter casing (e.g. "OpenBSD"). Although in the previous example, segmentation is trivial using letter casing, this was not always the case (e.g. "NASA"), so we had to manually annotate the data. The dataset is stored as plaintext file where each line is an example of space separated segments of a domain name. The examples are stored in their original letter casing, but harder and more interesting examples can be generated by lowercasing the input first."""

_HOMEPAGE = "https://github.com/google-research-datasets/common-crawl-domain-names"

_LICENSE = "MIT License"


# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URLs = {
    "train": "https://raw.githubusercontent.com/google-research-datasets/common-crawl-domain-names/master/data/train.txt",
    "test": "https://raw.githubusercontent.com/google-research-datasets/common-crawl-domain-names/master/data/test.txt",
    "dev": "https://raw.githubusercontent.com/google-research-datasets/common-crawl-domain-names/master/data/eval.txt",
}


class CrawlDomain(datasets.GeneratorBasedBuilder):
    """Corpus of domain names scraped from Common Crawl and manually annotated to add word boundaries (e.g. "commoncrawl" to "common crawl")."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):

        features = datasets.Features(
            {"example": datasets.Value("string")}  # These are the features of your dataset like images, labels ...
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        train_path = dl_manager.download_and_extract(_URLs["train"])
        test_path = dl_manager.download_and_extract(_URLs["test"])
        dev_path = dl_manager.download_and_extract(_URLs["dev"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": test_path, "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": dev_path,
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                yield id_, {
                    "example": row.rstrip(),
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Dutch Book Review Dataset"""


import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
The Dutch Book Review Dataset (DBRD) contains over 110k book reviews of which \
22k have associated binary sentiment polarity labels. It is intended as a \
benchmark for sentiment classification in Dutch and created due to a lack of \
annotated datasets in Dutch that are suitable for this task.
"""

_CITATION = """\
@article{DBLP:journals/corr/abs-1910-00896,
  author    = {Benjamin van der Burgh and
               Suzan Verberne},
  title     = {The merits of Universal Language Model Fine-tuning for Small Datasets
               - a case with Dutch book reviews},
  journal   = {CoRR},
  volume    = {abs/1910.00896},
  year      = {2019},
  url       = {http://arxiv.org/abs/1910.00896},
  archivePrefix = {arXiv},
  eprint    = {1910.00896},
  timestamp = {Fri, 04 Oct 2019 12:28:06 +0200},
  biburl    = {https://dblp.org/rec/journals/corr/abs-1910-00896.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DOWNLOAD_URL = "https://github.com/benjaminvdb/DBRD/releases/download/v3.0/DBRD_v3.tgz"


class DBRDConfig(datasets.BuilderConfig):
    """BuilderConfig for DBRD."""

    def __init__(self, **kwargs):
        """BuilderConfig for DBRD.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(DBRDConfig, self).__init__(version=datasets.Version("3.0.0", ""), **kwargs)


class DBRD(datasets.GeneratorBasedBuilder):
    """Dutch Book Review Dataset."""

    BUILDER_CONFIGS = [
        DBRDConfig(
            name="plain_text",
            description="Plain text",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"text": datasets.Value("string"), "label": datasets.features.ClassLabel(names=["neg", "pos"])}
            ),
            supervised_keys=None,
            homepage="https://github.com/benjaminvdb/DBRD",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        archive = dl_manager.download(_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"files": dl_manager.iter_archive(archive), "split": "train"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"files": dl_manager.iter_archive(archive), "split": "test"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split("unsupervised"),
                gen_kwargs={"files": dl_manager.iter_archive(archive), "split": "unsup", "labeled": False},
            ),
        ]

    def _generate_examples(self, files, split, labeled=True):
        """Generate DBRD examples."""
        # For labeled examples, extract the label from the path.
        if labeled:
            for path, f in files:
                if path.startswith(f"DBRD/{split}"):
                    label = {"pos": 1, "neg": 0}[path.split("/")[2]]
                    yield path, {"text": f.read().decode("utf-8"), "label": label}
        else:
            for path, f in files:
                if path.startswith(f"DBRD/{split}"):
                    yield path, {"text": f.read().decode("utf-8"), "label": -1}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""WiLI-2018, the Wikipedia language identification benchmark dataset"""


import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@dataset{thoma_martin_2018_841984,
  author       = {Thoma, Martin},
  title        = {{WiLI-2018 - Wikipedia Language Identification database}},
  month        = jan,
  year         = 2018,
  publisher    = {Zenodo},
  version      = {1.0.0},
  doi          = {10.5281/zenodo.841984},
  url          = {https://doi.org/10.5281/zenodo.841984}
}
"""

_DESCRIPTION = """\
It is a benchmark dataset for language identification and contains 235000 paragraphs of 235 languages
"""

# TODO: Add a link to an official homepage for the dataset here
_HOMEPAGE = "https://zenodo.org/record/841984"

# TODO: Add the licence for the dataset here if you can find it
_LICENSE = "ODC Open Database License v1.0"


# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_TRAIN_DOWNLOAD_URL = "https://drive.google.com/uc?export=download&id=1ZzlIQvw1KNBG97QQCfdatvVrrbeLaM1u"
_TEST_DOWNLOAD_URL = "https://drive.google.com/uc?export=download&id=1Xx4kFc1Xdzz8AhDasxZ0cSa-a35EQSDZ"

_CLASSES = [
    "cdo",
    "glk",
    "jam",
    "lug",
    "san",
    "rue",
    "wol",
    "new",
    "mwl",
    "bre",
    "ara",
    "hye",
    "xmf",
    "ext",
    "cor",
    "yor",
    "div",
    "asm",
    "lat",
    "cym",
    "hif",
    "ace",
    "kbd",
    "tgk",
    "rus",
    "nso",
    "mya",
    "msa",
    "ava",
    "cbk",
    "urd",
    "deu",
    "swa",
    "pus",
    "bxr",
    "udm",
    "csb",
    "yid",
    "vro",
    "por",
    "pdc",
    "eng",
    "tha",
    "hat",
    "lmo",
    "pag",
    "jav",
    "chv",
    "nan",
    "sco",
    "kat",
    "bho",
    "bos",
    "kok",
    "oss",
    "mri",
    "fry",
    "cat",
    "azb",
    "kin",
    "hin",
    "sna",
    "dan",
    "egl",
    "mkd",
    "ron",
    "bul",
    "hrv",
    "som",
    "pam",
    "nav",
    "ksh",
    "nci",
    "khm",
    "sgs",
    "srn",
    "bar",
    "cos",
    "ckb",
    "pfl",
    "arz",
    "roa-tara",
    "fra",
    "mai",
    "zh-yue",
    "guj",
    "fin",
    "kir",
    "vol",
    "hau",
    "afr",
    "uig",
    "lao",
    "swe",
    "slv",
    "kor",
    "szl",
    "srp",
    "dty",
    "nrm",
    "dsb",
    "ind",
    "wln",
    "pnb",
    "ukr",
    "bpy",
    "vie",
    "tur",
    "aym",
    "lit",
    "zea",
    "pol",
    "est",
    "scn",
    "vls",
    "stq",
    "gag",
    "grn",
    "kaz",
    "ben",
    "pcd",
    "bjn",
    "krc",
    "amh",
    "diq",
    "ltz",
    "ita",
    "kab",
    "bel",
    "ang",
    "mhr",
    "che",
    "koi",
    "glv",
    "ido",
    "fao",
    "bak",
    "isl",
    "bcl",
    "tet",
    "jpn",
    "kur",
    "map-bms",
    "tyv",
    "olo",
    "arg",
    "ori",
    "lim",
    "tel",
    "lin",
    "roh",
    "sqi",
    "xho",
    "mlg",
    "fas",
    "hbs",
    "tam",
    "aze",
    "lad",
    "nob",
    "sin",
    "gla",
    "nap",
    "snd",
    "ast",
    "mal",
    "mdf",
    "tsn",
    "nds",
    "tgl",
    "nno",
    "sun",
    "lzh",
    "jbo",
    "crh",
    "pap",
    "oci",
    "hak",
    "uzb",
    "zho",
    "hsb",
    "sme",
    "mlt",
    "vep",
    "lez",
    "nld",
    "nds-nl",
    "mrj",
    "spa",
    "ceb",
    "ina",
    "heb",
    "hun",
    "que",
    "kaa",
    "mar",
    "vec",
    "frp",
    "ell",
    "sah",
    "eus",
    "ces",
    "slk",
    "chr",
    "lij",
    "nep",
    "srd",
    "ilo",
    "be-tarask",
    "bod",
    "orm",
    "war",
    "glg",
    "mon",
    "gle",
    "min",
    "ibo",
    "ile",
    "epo",
    "lav",
    "lrc",
    "als",
    "mzn",
    "rup",
    "fur",
    "tat",
    "myv",
    "pan",
    "ton",
    "kom",
    "wuu",
    "tcy",
    "tuk",
    "kan",
    "ltg",
]


class Wili_2018(datasets.GeneratorBasedBuilder):
    """WiLI Language Identification Dataset"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="WiLI-2018 dataset",
            version=VERSION,
            description="Plain text of import of WiLI-2018",
        )
    ]

    def _info(self):

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=datasets.Features(
                {"sentence": datasets.Value("string"), "label": datasets.features.ClassLabel(names=_CLASSES)}
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="sentence", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        test_path = dl_manager.download_and_extract(_TEST_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):

        with open(filepath, encoding="utf-8") as f:
            for id_, line in enumerate(f):
                text, label = line.rsplit(",", 1)
                text = text.strip('"')
                label = int(label.strip())
                yield id_, {"sentence": text, "label": label - 1}



import json
import os

import datasets


_CITATION = """\
@misc{xiao2018cail2018,
      title={CAIL2018: A Large-Scale Legal Dataset for Judgment Prediction},
      author={Chaojun Xiao and Haoxi Zhong and Zhipeng Guo and Cunchao Tu and Zhiyuan Liu and Maosong Sun and Yansong Feng and Xianpei Han and Zhen Hu and Heng Wang and Jianfeng Xu},
      year={2018},
      eprint={1807.02478},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
In this paper, we introduce Chinese AI and Law challenge dataset (CAIL2018),
the first large-scale Chinese legal dataset for judgment prediction. CAIL contains more than 2.6 million
criminal cases published by the Supreme People's Court of China, which are several times larger than other
datasets in existing works on judgment prediction. Moreover, the annotations of judgment results are more
detailed and rich. It consists of applicable law articles, charges, and prison terms, which are expected
to be inferred according to the fact descriptions of cases. For comparison, we implement several conventional
text classification baselines for judgment prediction and experimental results show that it is still a
challenge for current models to predict the judgment results of legal cases, especially on prison terms.
To help the researchers make improvements on legal judgment prediction.
"""
_URL = "https://cail.oss-cn-qingdao.aliyuncs.com/CAIL2018_ALL_DATA.zip"


class Cail2018(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    def _info(self):
        features = datasets.Features(
            {
                "fact": datasets.Value("string"),
                "relevant_articles": datasets.Sequence(datasets.Value("int32")),
                "accusation": datasets.Sequence(datasets.Value("string")),
                "punish_of_money": datasets.Value("float"),
                "criminals": datasets.Sequence(datasets.Value("string")),
                "death_penalty": datasets.Value("bool"),
                "imprisonment": datasets.Value("float"),
                "life_imprisonment": datasets.Value("bool"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage="https://arxiv.org/abs/1807.02478",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dl_dir = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split("exercise_contest_train"),
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "final_all_data/exercise_contest/data_train.json"),
                    "split": "exercise_contest_train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("exercise_contest_valid"),
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "final_all_data/exercise_contest/data_valid.json"),
                    "split": "exercise_contest_valid",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("exercise_contest_test"),
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "final_all_data/exercise_contest/data_test.json"),
                    "split": "exercise_contest_test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("first_stage_train"),
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "final_all_data/first_stage/train.json"),
                    "split": "first_stage_train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("first_stage_test"),
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "final_all_data/first_stage/test.json"),
                    "split": "first_stage_test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("final_test"),
                gen_kwargs={"filepath": os.path.join(dl_dir, "final_all_data/final_test.json"), "split": "final_test"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for idx, row in enumerate(f):
                data = json.loads(row)
                yield idx, {
                    "fact": data["fact"],
                    "relevant_articles": data["meta"]["relevant_articles"],
                    "accusation": data["meta"]["accusation"],
                    "punish_of_money": data["meta"]["punish_of_money"],
                    "criminals": data["meta"]["criminals"],
                    "death_penalty": data["meta"]["term_of_imprisonment"]["death_penalty"],
                    "imprisonment": data["meta"]["term_of_imprisonment"]["imprisonment"],
                    "life_imprisonment": data["meta"]["term_of_imprisonment"]["life_imprisonment"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import datasets


_CITATION = """\
@InProceedings{hindencorp05:lrec:2014,
  author = {Ond{\v{r}}ej Bojar and Vojt{\v{e}}ch Diatka
            and Pavel Rychl{\'{y}} and Pavel Stra{\v{n}}{\'{a}}k
            and V{\'{}}t Suchomel and Ale{\v{s}} Tamchyna and Daniel Zeman},
  title = "{HindEnCorp - Hindi-English and Hindi-only Corpus for Machine
            Translation}",
  booktitle = {Proceedings of the Ninth International Conference on Language
               Resources and Evaluation (LREC'14)},
  year = {2014},
  month = {may},
  date = {26-31},
  address = {Reykjavik, Iceland},
  editor = {Nicoletta Calzolari (Conference Chair) and Khalid Choukri and
     Thierry Declerck and Hrafn Loftsson and Bente Maegaard and Joseph Mariani
     and Asuncion Moreno and Jan Odijk and Stelios Piperidis},
  publisher = {European Language Resources Association (ELRA)},
  isbn = {978-2-9517408-8-4},
  language = {english}
}
"""

_DESCRIPTION = """\
HindEnCorp parallel texts (sentence-aligned) come from the following sources:
Tides, which contains 50K sentence pairs taken mainly from news articles. This dataset was originally col- lected for the DARPA-TIDES surprise-language con- test in 2002, later refined at IIIT Hyderabad and provided for the NLP Tools Contest at ICON 2008 (Venkatapathy, 2008).

Commentaries by Daniel Pipes contain 322 articles in English written by a journalist Daniel Pipes and translated into Hindi.

EMILLE. This corpus (Baker et al., 2002) consists of three components: monolingual, parallel and annotated corpora. There are fourteen monolingual sub- corpora, including both written and (for some lan- guages) spoken data for fourteen South Asian lan- guages. The EMILLE monolingual corpora contain in total 92,799,000 words (including 2,627,000 words of transcribed spoken data for Bengali, Gujarati, Hindi, Punjabi and Urdu). The parallel corpus consists of 200,000 words of text in English and its accompanying translations into Hindi and other languages.

Smaller datasets as collected by Bojar et al. (2010) include the corpus used at ACL 2005 (a subcorpus of EMILLE), a corpus of named entities from Wikipedia (crawled in 2009), and Agriculture domain parallel corpus.
￼
For the current release, we are extending the parallel corpus using these sources:
Intercorp (Čermák and Rosen,2012) is a large multilingual parallel corpus of 32 languages including Hindi. The central language used for alignment is Czech. Intercorp’s core texts amount to 202 million words. These core texts are most suitable for us because their sentence alignment is manually checked and therefore very reliable. They cover predominately short sto- ries and novels. There are seven Hindi texts in Inter- corp. Unfortunately, only for three of them the English translation is available; the other four are aligned only with Czech texts. The Hindi subcorpus of Intercorp contains 118,000 words in Hindi.

TED talks 3 held in various languages, primarily English, are equipped with transcripts and these are translated into 102 languages. There are 179 talks for which Hindi translation is available.

The Indic multi-parallel corpus (Birch et al., 2011; Post et al., 2012) is a corpus of texts from Wikipedia translated from the respective Indian language into English by non-expert translators hired over Mechanical Turk. The quality is thus somewhat mixed in many respects starting from typesetting and punctuation over capi- talization, spelling, word choice to sentence structure. A little bit of control could be in principle obtained from the fact that every input sentence was translated 4 times. We used the 2012 release of the corpus.

Launchpad.net is a software collaboration platform that hosts many open-source projects and facilitates also collaborative localization of the tools. We downloaded all revisions of all the hosted projects and extracted the localization (.po) files.

Other smaller datasets. This time, we added Wikipedia entities as crawled in 2013 (including any morphological variants of the named entitity that appears on the Hindi variant of the Wikipedia page) and words, word examples and quotes from the Shabdkosh online dictionary.
"""

_HOMEPAGE = "https://lindat.mff.cuni.cz/repository/xmlui/handle/11858/00-097C-0000-0023-625F-0"

_LICENSE = "CC BY-NC-SA 3.0"

_URLs = "https://lindat.mff.cuni.cz/repository/xmlui/bitstream/handle/11858/00-097C-0000-0023-625F-0/hindencorp05.plaintext.gz?sequence=3&isAllowed=y"


class HindEncorp(datasets.GeneratorBasedBuilder):
    """Short description of my dataset."""

    def _info(self):

        features = datasets.Features(
            {
                "id": datasets.Value("string"),
                "source": datasets.Value("string"),
                "alignment_type": datasets.Value("string"),
                "alignment_quality": datasets.Value("string"),
                "translation": datasets.features.Translation(languages=["en", "hi"]),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URLs)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as f:
            for id_, line in enumerate(f):
                splits = line.strip().split("\t")
                yield id_, {
                    "id": str(id_),
                    "source": splits[0],
                    "alignment_type": splits[1],
                    "alignment_quality": splits[2],
                    "translation": {"en": splits[3], "hi": splits[4]},
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TaPaCo: A Corpus of Sentential Paraphrases for 73 Languages"""


import csv
import os

import datasets


_CITATION = """\
@dataset{scherrer_yves_2020_3707949,
  author       = {Scherrer, Yves},
  title        = {{TaPaCo: A Corpus of Sentential Paraphrases for 73 Languages}},
  month        = mar,
  year         = 2020,
  publisher    = {Zenodo},
  version      = {1.0},
  doi          = {10.5281/zenodo.3707949},
  url          = {https://doi.org/10.5281/zenodo.3707949}
}
"""


_DESCRIPTION = """\
A freely available paraphrase corpus for 73 languages extracted from the Tatoeba database. \
Tatoeba is a crowdsourcing project mainly geared towards language learners. Its aim is to provide example sentences \
and translations for particular linguistic constructions and words. The paraphrase corpus is created by populating a \
graph with Tatoeba sentences and equivalence links between sentences “meaning the same thing”. This graph is then \
traversed to extract sets of paraphrases. Several language-independent filters and pruning steps are applied to \
remove uninteresting sentences. A manual evaluation performed on three languages shows that between half and three \
quarters of inferred paraphrases are correct and that most remaining ones are either correct but trivial, \
or near-paraphrases that neutralize a morphological distinction. The corpus contains a total of 1.9 million \
sentences, with 200 – 250 000 sentences per language. It covers a range of languages for which, to our knowledge,\
no other paraphrase dataset exists."""


_HOMEPAGE = "https://zenodo.org/record/3707949#.X9Dh0cYza3I"


_LICENSE = "Creative Commons Attribution 2.0 Generic"


_URLs = {
    "train": "https://zenodo.org/record/3707949/files/tapaco_v1.0.zip?download=1",
}

_VERSION = "1.0.0"
_LANGUAGES = {
    "af": "Afrikaans",
    "ar": "Arabic",
    "az": "Azerbaijani",
    "be": "Belarusian",
    "ber": "Berber languages",
    "bg": "Bulgarian",
    "bn": "Bengali",
    "br": "Breton",
    "ca": "Catalan; Valencian",
    "cbk": "Chavacano",
    "cmn": "Mandarin",
    "cs": "Czech",
    "da": "Danish",
    "de": "German",
    "el": "Greek, Modern (1453-)",
    "en": "English",
    "eo": "Esperanto",
    "es": "Spanish; Castilian",
    "et": "Estonian",
    "eu": "Basque",
    "fi": "Finnish",
    "fr": "French",
    "gl": "Galician",
    "gos": "Gronings",
    "he": "Hebrew",
    "hi": "Hindi",
    "hr": "Croatian",
    "hu": "Hungarian",
    "hy": "Armenian",
    "ia": "Interlingua (International Auxiliary Language Association)",
    "id": "Indonesian",
    "ie": "Interlingue; Occidental",
    "io": "Ido",
    "is": "Icelandic",
    "it": "Italian",
    "ja": "Japanese",
    "jbo": "Lojban",
    "kab": "Kabyle",
    "ko": "Korean",
    "kw": "Cornish",
    "la": "Latin",
    "lfn": "Lingua Franca Nova\t",
    "lt": "Lithuanian",
    "mk": "Macedonian",
    "mr": "Marathi",
    "nb": "Bokmål, Norwegian; Norwegian Bokmål",
    "nds": "Low German; Low Saxon; German, Low; Saxon, Low",
    "nl": "Dutch; Flemish",
    "orv": "Old Russian",
    "ota": "Turkish, Ottoman (1500-1928)",
    "pes": "Iranian Persian",
    "pl": "Polish",
    "pt": "Portuguese",
    "rn": "Rundi",
    "ro": "Romanian; Moldavian; Moldovan",
    "ru": "Russian",
    "sl": "Slovenian",
    "sr": "Serbian",
    "sv": "Swedish",
    "tk": "Turkmen",
    "tl": "Tagalog",
    "tlh": "Klingon; tlhIngan-Hol",
    "toki": "Toki Pona",
    "tr": "Turkish",
    "tt": "Tatar",
    "ug": "Uighur; Uyghur",
    "uk": "Ukrainian",
    "ur": "Urdu",
    "vi": "Vietnamese",
    "vo": "Volapük",
    "war": "Waray",
    "wuu": "Wu Chinese",
    "yue": "Yue Chinese",
}
_ALL_LANGUAGES = "all_languages"


class TapacoConfig(datasets.BuilderConfig):
    """BuilderConfig for TapacoConfig."""

    def __init__(self, languages=None, **kwargs):
        super(TapacoConfig, self).__init__(version=datasets.Version(_VERSION, ""), **kwargs),
        self.languages = languages


class Tapaco(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        TapacoConfig(
            name=_ALL_LANGUAGES,
            languages=_LANGUAGES,
            description="A collection of paraphrase corpus for 73 languages to aid paraphrase "
            "detection and generation.",
        )
    ] + [
        TapacoConfig(
            name=lang,
            languages=[lang],
            description=f"{_LANGUAGES[lang]} A collection of paraphrase corpus for 73 languages to "
            f"aid paraphrase "
            "detection and generation.",
        )
        for lang in _LANGUAGES
    ]
    BUILDER_CONFIG_CLASS = TapacoConfig
    DEFAULT_CONFIG_NAME = _ALL_LANGUAGES

    def _info(self):
        features = datasets.Features(
            {
                "paraphrase_set_id": datasets.Value("string"),
                "sentence_id": datasets.Value("string"),
                "paraphrase": datasets.Value("string"),
                "lists": datasets.Sequence(datasets.Value("string")),
                "tags": datasets.Sequence(datasets.Value("string")),
                "language": datasets.Value("string"),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"data_dir": data_dir["train"]},
            ),
        ]

    def _generate_examples(self, data_dir):
        """Yields examples."""
        base_path = os.path.join(data_dir, "tapaco_v1.0")
        file_dict = {lang: os.path.join(base_path, lang + ".txt") for lang in self.config.languages}
        id_ = -1
        for language, filepath in file_dict.items():
            with open(filepath, encoding="utf-8") as csv_file:
                csv_reader = csv.reader(
                    csv_file, quotechar='"', delimiter="\t", quoting=csv.QUOTE_ALL, skipinitialspace=True
                )
                for row in csv_reader:
                    id_ += 1
                    paraphrase_set_id, sentence_id, paraphrase, lists, tags = row[: len(row)] + [""] * (5 - len(row))
                    yield id_, {
                        "paraphrase_set_id": paraphrase_set_id,
                        "sentence_id": sentence_id,
                        "paraphrase": paraphrase,
                        "lists": lists.split(";"),
                        "tags": tags.split(";"),
                        "language": language,
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""WUT Relations Between Sentences Corpus"""


import csv

import datasets


# TODO: Add BibTeX citation
# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@misc{11321/305,
 title = {{WUT} Relations Between Sentences Corpus},
 author = {Oleksy, Marcin and Fikus, Dominika and Wolski, Michal and Podbielska, Malgorzata and Turek, Agnieszka and Kędzia, Pawel},
 url = {http://hdl.handle.net/11321/305},
 note = {{CLARIN}-{PL} digital repository},
 copyright = {Attribution-{ShareAlike} 3.0 Unported ({CC} {BY}-{SA} 3.0)},
 year = {2016}
}
"""

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
WUT Relations Between Sentences Corpus contains 2827 pairs of related sentences.
Relationships are derived from Cross-document Structure Theory (CST), which enables multi-document summarization through identification of cross-document rhetorical relationships within a cluster of related documents.
Every relation was marked by at least 3 annotators.
"""

# TODO: Add a link to an official homepage for the dataset here
_HOMEPAGE = "https://clarin-pl.eu/dspace/handle/11321/305"

# TODO: Add the licence for the dataset here if you can find it
_LICENSE = "Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0)"

# TODO: Add link to the official dataset URLs here
# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "https://clarin-pl.eu/dspace/bitstream/handle/11321/305/sem_rels-betw-sents.csv"


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class Wrbsc(datasets.GeneratorBasedBuilder):
    """WUT Relations Between Sentences Corpus"""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        # TODO: This method specifies the datasets.DatasetInfo object which contains informations and typings for the dataset
        features = datasets.Features(
            {
                "sentence1": datasets.Value("string"),
                "sentence2": datasets.Value("string"),
                "relationship": datasets.ClassLabel(
                    names=[
                        "Krzyżowanie_się",
                        "Tło_historyczne",
                        "Źródło",
                        "Dalsze_informacje",
                        "Zawieranie",
                        "Opis",
                        "Uszczegółowienie",
                        "Parafraza",
                        "Spełnienie",
                        "Mowa_zależna",
                        "Zmiana_poglądu",
                        "Streszczenie",
                        "Tożsamość",
                        "Sprzeczność",
                        "Modalność",
                        "Cytowanie",
                    ]
                ),
            }
        )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs
        # It can accept any type or nested list/dict and will give back the same structure with the url replaced with path to local files.
        # By default the archives will be extracted and a path to a cached folder where they are extracted is returned instead of the archive
        filepath = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": filepath,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        # TODO: This method will receive as arguments the `gen_kwargs` defined in the previous `_split_generators` method.
        # It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        # The key is not important, it's more here for legacy reason (legacy from tfds)

        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(
                f, delimiter="\t", fieldnames=["0", "1", "s1", "s2", "r", "2"], quoting=csv.QUOTE_NONE
            )
            for idx, row in enumerate(reader):
                yield idx, {
                    "sentence1": row["s1"][1:-1],
                    "sentence2": row["s2"][1:-1],
                    "relationship": row["r"],
                }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Universal Declaration of Human Rights"""


import glob
import os
import xml.etree.ElementTree as ET

import datasets


_DESCRIPTION = """\
The Universal Declaration of Human Rights (UDHR) is a milestone document in the history of human rights. Drafted by
representatives with different legal and cultural backgrounds from all regions of the world, it set out, for the
first time, fundamental human rights to be universally protected. The Declaration was adopted by the UN General
Assembly in Paris on 10 December 1948 during its 183rd plenary meeting. The dataset includes translations of the
document in 464 languages and dialects.

© 1996 – 2009 The Office of the High Commissioner for Human Rights

This plain text version prepared by the “UDHR in Unicode” project, https://www.unicode.org/udhr.
"""

_WEBPAGE = "https://www.ohchr.org/EN/UDHR/Pages/UDHRIndex.aspx"

# XML for meta-info about language and TXT for text
_XML_DOWNLOAD_URL = "https://unicode.org/udhr/assemblies/udhr_xml.zip"
_TXT_DOWNLOAD_URL = "https://unicode.org/udhr/assemblies/udhr_txt.zip"


class UDHN(datasets.GeneratorBasedBuilder):
    """Universal Declaration of Human Rights"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "lang_key": datasets.Value("string"),
                    "lang_name": datasets.Value("string"),
                    "iso639-3": datasets.Value("string"),
                    "iso15924": datasets.Value("string"),
                }
            ),
            homepage=_WEBPAGE,
        )

    def _split_generators(self, dl_manager):
        xml_dir = dl_manager.download_and_extract(_XML_DOWNLOAD_URL)
        txt_dir = dl_manager.download_and_extract(_TXT_DOWNLOAD_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"xml_dir": xml_dir, "txt_dir": txt_dir}),
        ]

    def _generate_examples(self, xml_dir, txt_dir):
        xml_paths = glob.glob(os.path.join(xml_dir, "udhr_*.xml"))
        for i, xml_path in enumerate(xml_paths):
            lang_attr = ET.parse(xml_path).getroot().attrib
            lang = lang_attr["key"]

            txt_path = os.path.join(txt_dir, f"udhr_{lang}.txt")
            assert os.path.exists(txt_path), f"xml file {xml_path} found with no corresponding txt file"

            with open(txt_path, "r", encoding="utf-8") as f:
                lines = f.readlines()[6:]  # ignore first 6 lines (metainfo)
            text = "\n".join([line.strip() for line in lines if len(line.strip()) > 0])

            yield i, {
                "text": text,
                "lang_key": lang,
                "lang_name": lang_attr["n"],
                "iso639-3": lang_attr["iso639-3"],
                "iso15924": lang_attr["iso15924"],
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Indo-Aryan Language Identification Shared Task Dataset"""


import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@proceedings{ws-2018-nlp-similar,
    title = "Proceedings of the Fifth Workshop on {NLP} for Similar Languages, Varieties and Dialects ({V}ar{D}ial 2018)",
    editor = {Zampieri, Marcos  and
      Nakov, Preslav  and
      Ljube{\v{s}}i{\'c}, Nikola  and
      Tiedemann, J{\"o}rg  and
      Malmasi, Shervin  and
      Ali, Ahmed},
    month = aug,
    year = "2018",
    address = "Santa Fe, New Mexico, USA",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W18-3900",
}
"""

_DESCRIPTION = """\
This dataset is introduced in a task which aimed at identifying 5 closely-related languages of Indo-Aryan language family –
Hindi (also known as Khari Boli), Braj Bhasha, Awadhi, Bhojpuri, and Magahi.
"""

_URL = "https://raw.githubusercontent.com/kmi-linguistics/vardial2018/master/dataset/{}.txt"


class Ilist(datasets.GeneratorBasedBuilder):
    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "language_id": datasets.ClassLabel(names=["AWA", "BRA", "MAG", "BHO", "HIN"]),
                    "text": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/kmi-linguistics/vardial2018",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="language_id")],
        )

    def _split_generators(self, dl_manager):
        filepaths = dl_manager.download_and_extract(
            {
                "train": _URL.format("train"),
                "test": _URL.format("gold"),
                "dev": _URL.format("dev"),
            }
        )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": filepaths["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": filepaths["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": filepaths["dev"],
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, "r", encoding="utf-8") as file:
            for idx, row in enumerate(file):
                row = row.strip("\n").split("\t")
                if len(row) == 1:
                    continue
                yield idx, {"language_id": row[1], "text": row[0]}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The United Nations Parallel Corpus v1.0"""


import itertools
import os

import datasets


_CITATION = """\
@inproceedings{ziemski-etal-2016-united,
    title = "The {U}nited {N}ations Parallel Corpus v1.0",
    author = "Ziemski, Micha{\\l}  and
      Junczys-Dowmunt, Marcin  and
      Pouliquen, Bruno",
    booktitle = "Proceedings of the Tenth International Conference on Language Resources and Evaluation ({LREC}'16)",
    month = may,
    year = "2016",
    address = "Portoro{\v{z}}, Slovenia",
    publisher = "European Language Resources Association (ELRA)",
    url = "https://www.aclweb.org/anthology/L16-1561",
    pages = "3530--3534",
    abstract = "This paper describes the creation process and statistics of the official United Nations Parallel Corpus, the first parallel corpus composed from United Nations documents published by the original data creator. The parallel corpus presented consists of manually translated UN documents from the last 25 years (1990 to 2014) for the six official UN languages, Arabic, Chinese, English, French, Russian, and Spanish. The corpus is freely available for download under a liberal license. Apart from the pairwise aligned documents, a fully aligned subcorpus for the six official UN languages is distributed. We provide baseline BLEU scores of our Moses-based SMT systems trained with the full data of language pairs involving English and for all possible translation directions of the six-way subcorpus.",
}
"""


_DESCRIPTION = """\
This parallel corpus consists of manually translated UN documents from the last 25 years (1990 to 2014) \
for the six official UN languages, Arabic, Chinese, English, French, Russian, and Spanish.
"""


_HOMEPAGE = "http://opus.nlpl.eu/UNPC.php"

_LANGUAGES = ["ar", "en", "es", "fr", "ru", "zh"]
_LANGUAGE_PAIRS = list(itertools.combinations(_LANGUAGES, 2))

_BASE_URL = "http://opus.nlpl.eu/download.php?f=UNPC/v1.0/moses"
_URLS = {f"{l1}-{l2}": f"{_BASE_URL}/{l1}-{l2}.txt.zip" for l1, l2 in _LANGUAGE_PAIRS}


class UnPc(datasets.GeneratorBasedBuilder):
    """The United Nations Parallel Corpus v1.0"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name=f"{l1}-{l2}", version=datasets.Version("1.0.0"), description=f"UNPC {l1}-{l2}")
        for l1, l2 in _LANGUAGE_PAIRS
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=tuple(self.config.name.split("-")))}
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        lang_pair = self.config.name.split("-")
        data_dir = dl_manager.download_and_extract(_URLS[self.config.name])
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "source_file": os.path.join(data_dir, f"UNPC.{self.config.name}.{lang_pair[0]}"),
                    "target_file": os.path.join(data_dir, f"UNPC.{self.config.name}.{lang_pair[1]}"),
                },
            ),
        ]

    def _generate_examples(self, source_file, target_file):
        source, target = tuple(self.config.name.split("-"))
        with open(source_file, encoding="utf-8") as src_f, open(target_file, encoding="utf-8") as tgt_f:
            for idx, (l1, l2) in enumerate(zip(src_f, tgt_f)):
                result = {"translation": {source: l1.strip(), target: l2.strip()}}
                yield idx, result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""DaNE: named entity annotation for the Danish Universal Dependencies
treebank using the CoNLL-2003 annotation scheme."""


import os

import datasets


_CITATION = """\
@inproceedings{hvingelby-etal-2020-dane,
    title = "{D}a{NE}: A Named Entity Resource for {D}anish",
    author = "Hvingelby, Rasmus  and
      Pauli, Amalie Brogaard  and
      Barrett, Maria  and
      Rosted, Christina  and
      Lidegaard, Lasse Malm  and
      Søgaard, Anders",
    booktitle = "Proceedings of the 12th Language Resources and Evaluation Conference",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://www.aclweb.org/anthology/2020.lrec-1.565",
    pages = "4597--4604",
    abstract = "We present a named entity annotation for the Danish Universal Dependencies treebank using the CoNLL-2003 annotation scheme: DaNE. It is the largest publicly available, Danish named entity gold annotation. We evaluate the quality of our annotations intrinsically by double annotating the entire treebank and extrinsically by comparing our annotations to a recently released named entity annotation of the validation and test sections of the Danish Universal Dependencies treebank. We benchmark the new resource by training and evaluating competitive architectures for supervised named entity recognition (NER), including FLAIR, monolingual (Danish) BERT and multilingual BERT. We explore cross-lingual transfer in multilingual BERT from five related languages in zero-shot and direct transfer setups, and we show that even with our modestly-sized training set, we improve Danish NER over a recent cross-lingual approach, as well as over zero-shot transfer from five related languages. Using multilingual BERT, we achieve higher performance by fine-tuning on both DaNE and a larger Bokm{\aa}l (Norwegian) training set compared to only using DaNE. However, the highest performance isachieved by using a Danish BERT fine-tuned on DaNE. Our dataset enables improvements and applicability for Danish NER beyond cross-lingual methods. We employ a thorough error analysis of the predictions of the best models for seen and unseen entities, as well as their robustness on un-capitalized text. The annotated dataset and all the trained models are made publicly available.",
    language = "English",
    ISBN = "979-10-95546-34-4",
}
"""

_DESCRIPTION = """\
The DaNE dataset has been annotated with Named Entities for PER, ORG and LOC
by the Alexandra Institute.
It is a reannotation of the UD-DDT (Universal Dependency - Danish Dependency Treebank)
which has annotations for dependency parsing and part-of-speech (POS) tagging.
The Danish UD treebank (Johannsen et al., 2015, UD-DDT) is a conversion of
the Danish Dependency Treebank (Buch-Kromann et al. 2003) based on texts
from Parole (Britt, 1998).
"""

_HOMEPAGE = "https://github.com/alexandrainst/danlp/blob/master/docs/docs/datasets.md#dane"

_LICENSE = "CC BY-SA 4.0"

# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "http://danlp-downloads.alexandra.dk/datasets/ddt.zip"


class Dane(datasets.GeneratorBasedBuilder):
    """DaNE dataset"""

    def _info(self):
        features = datasets.Features(
            {
                "sent_id": datasets.Value("string"),
                "text": datasets.Value("string"),
                "tok_ids": datasets.Sequence(datasets.Value("int64")),
                "tokens": datasets.Sequence(datasets.Value("string")),
                "lemmas": datasets.Sequence(datasets.Value("string")),
                "pos_tags": datasets.Sequence(
                    datasets.features.ClassLabel(
                        names=[
                            "NUM",
                            "CCONJ",
                            "PRON",
                            "VERB",
                            "INTJ",
                            "AUX",
                            "ADJ",
                            "PROPN",
                            "PART",
                            "ADV",
                            "PUNCT",
                            "ADP",
                            "NOUN",
                            "X",
                            "DET",
                            "SYM",
                            "SCONJ",
                        ]
                    )
                ),
                "morph_tags": datasets.Sequence(datasets.Value("string")),
                "dep_ids": datasets.Sequence(datasets.Value("int64")),
                "dep_labels": datasets.Sequence(
                    datasets.ClassLabel(
                        names=[
                            "parataxis",
                            "mark",
                            "nummod",
                            "discourse",
                            "compound:prt",
                            "reparandum",
                            "vocative",
                            "list",
                            "obj",
                            "dep",
                            "det",
                            "obl:loc",
                            "flat",
                            "iobj",
                            "cop",
                            "expl",
                            "obl",
                            "conj",
                            "nmod",
                            "root",
                            "acl:relcl",
                            "goeswith",
                            "appos",
                            "fixed",
                            "obl:tmod",
                            "xcomp",
                            "advmod",
                            "nmod:poss",
                            "aux",
                            "ccomp",
                            "amod",
                            "cc",
                            "advcl",
                            "nsubj",
                            "punct",
                            "case",
                        ]
                    )
                ),
                "ner_tags": datasets.Sequence(
                    datasets.features.ClassLabel(
                        names=[
                            "O",
                            "B-PER",
                            "I-PER",
                            "B-ORG",
                            "I-ORG",
                            "B-LOC",
                            "I-LOC",
                            "B-MISC",
                            "I-MISC",
                        ]
                    )
                ),
            }
        )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs
        # It can accept any type or nested list/dict and will give back the same structure with the url replaced with path to local files.
        # By default the archives will be extracted and a path to a cached folder where they are extracted is returned instead of the archive
        data_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "ddt.train.conllu"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, "ddt.test.conllu"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "ddt.dev.conllu"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as f:
            guid = 0
            sent_id = ""
            text = ""
            tok_ids = []
            tokens = []
            lemmas = []
            pos_tags = []
            morph_tags = []
            dephead_ids = []
            dep_labels = []
            ner_tags = []
            for line in f:
                if line.startswith("#"):
                    var, val = line.split(" = ", maxsplit=1)
                    var = var.replace("# ", "")
                    if var == "sent_id":
                        sent_id = val
                    elif var == "text":
                        text = val
                elif line == "" or line == "\n":
                    if tokens:
                        yield guid, {
                            "sent_id": sent_id,
                            "text": text,
                            "tok_ids": tok_ids,
                            "tokens": tokens,
                            "lemmas": lemmas,
                            "pos_tags": pos_tags,
                            "morph_tags": morph_tags,
                            "dep_ids": dephead_ids,
                            "dep_labels": dep_labels,
                            "ner_tags": ner_tags,
                        }
                        guid += 1
                        sent_id = ""
                        text = ""
                        tok_ids = []
                        tokens = []
                        lemmas = []
                        pos_tags = []
                        morph_tags = []
                        dephead_ids = []
                        dep_labels = []
                        ner_tags = []
                else:
                    # conllu tokens tab space separated
                    splits = line.split("\t")
                    tok_ids.append(int(splits[0]))
                    tokens.append(splits[1])
                    lemmas.append(splits[2])
                    pos_tags.append(splits[3])
                    morph_tags.append(splits[5])
                    dephead_ids.append(splits[6])
                    dep_labels.append(splits[7])
                    ner_tags.append(splits[9].rstrip().replace("name=", "").split("|")[0])
            # last example
            if tok_ids:
                yield guid, {
                    "sent_id": sent_id,
                    "text": text,
                    "tok_ids": tok_ids,
                    "tokens": tokens,
                    "lemmas": lemmas,
                    "pos_tags": pos_tags,
                    "morph_tags": morph_tags,
                    "dep_ids": dephead_ids,
                    "dep_labels": dep_labels,
                    "ner_tags": ner_tags,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""NLI-TR: The Turkish translation of SNLI and MultiNLI datasets using Amazon Translate."""


import codecs
import json
import os

import datasets


_CITATION = """\
@inproceedings{budur-etal-2020-data,
    title = "Data and Representation for Turkish Natural Language Inference",
    author = "Budur, Emrah and
      \"{O}zçelik, Rıza and
      G\"{u}ng\"{o}r, Tunga",
    booktitle = "Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP)",
    month = nov,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    abstract = "Large annotated datasets in NLP are overwhelmingly in English. This is an obstacle to progress in other languages. Unfortunately, obtaining new annotated resources for each task in each language would be prohibitively expensive. At the same time, commercial machine translation systems are now robust. Can we leverage these systems to translate English-language datasets automatically? In this paper, we offer a positive response for natural language inference (NLI) in Turkish. We translated two large English NLI datasets into Turkish and had a team of experts validate their translation quality and fidelity to the original labels. Using these datasets, we address core issues of representation for Turkish NLI. We find that in-language embeddings are essential and that morphological parsing can be avoided where the training set is large. Finally, we show that models trained on our machine-translated datasets are successful on human-translated evaluation sets. We share all code, models, and data publicly.",
}
"""

_DESCRIPTION = """\
The Natural Language Inference in Turkish (NLI-TR) is a set of two large scale datasets that were obtained by translating the foundational NLI corpora (SNLI and MNLI) using Amazon Translate.
"""

_HOMEPAGE = "https://github.com/boun-tabi/NLI-TR"


class NLITRConfig(datasets.BuilderConfig):
    """BuilderConfig for NLI-TR"""

    def __init__(self, version=None, data_url=None, **kwargs):
        super(NLITRConfig, self).__init__(version=datasets.Version(version, ""), **kwargs)
        self.data_url = data_url


class NliTr(datasets.GeneratorBasedBuilder):
    """NLI-TR: The Turkish translation of SNLI and MultiNLI datasets using Amazon Translate."""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIG_CLASS = NLITRConfig
    BUILDER_CONFIGS = [
        NLITRConfig(
            name="snli_tr",
            version="1.0.0",
            data_url="https://tabilab.cmpe.boun.edu.tr/datasets/nli_datasets/snli_tr_1.0.zip",
        ),
        NLITRConfig(
            name="multinli_tr",
            version="1.0.0",
            data_url="https://tabilab.cmpe.boun.edu.tr/datasets/nli_datasets/multinli_tr_1.0.zip",
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "idx": datasets.Value("int32"),
                "premise": datasets.Value("string"),
                "hypothesis": datasets.Value("string"),
                "label": datasets.features.ClassLabel(names=["entailment", "neutral", "contradiction"]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dl_dir = dl_manager.download_and_extract(self.config.data_url)
        data_dir = os.path.join(dl_dir)

        if self.config.name == "multinli_tr":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "multinli_tr_1.0_train.jsonl"),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name="validation_matched",
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "multinli_tr_1.0_dev_matched.jsonl"),
                        "split": "dev",
                    },
                ),
                datasets.SplitGenerator(
                    name="validation_mismatched",
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "multinli_tr_1.0_dev_mismatched.jsonl"),
                        "split": "dev",
                    },
                ),
            ]

        if self.config.name == "snli_tr":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "snli_tr_1.0_train.jsonl"),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "snli_tr_1.0_dev.jsonl"),
                        "split": "validation",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "snli_tr_1.0_test.jsonl"),
                        "split": "test",
                    },
                ),
            ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with codecs.open(filepath, encoding="utf-8") as f:
            for idx, row in enumerate(f):
                data = json.loads(row)
                example = {"idx": idx, "premise": data["sentence1"], "hypothesis": data["sentence2"]}

                if "gold_label" in data:
                    if data["gold_label"] != "-":
                        example["label"] = data["gold_label"]
                    else:
                        example["label"] = -1

                yield idx, example



"""Wisesight Sentiment Corpus: Social media messages in Thai language with sentiment category (positive, neutral, negative, question)"""


import json
import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@software{bact_2019_3457447,
  author       = {Suriyawongkul, Arthit and
                  Chuangsuwanich, Ekapol and
                  Chormai, Pattarawat and
                  Polpanumas, Charin},
  title        = {PyThaiNLP/wisesight-sentiment: First release},
  month        = sep,
  year         = 2019,
  publisher    = {Zenodo},
  version      = {v1.0},
  doi          = {10.5281/zenodo.3457447},
  url          = {https://doi.org/10.5281/zenodo.3457447}
}
"""

_DESCRIPTION = """\
Wisesight Sentiment Corpus: Social media messages in Thai language with sentiment category (positive, neutral, negative, question)
* Released to public domain under Creative Commons Zero v1.0 Universal license.
* Category (Labels): {"pos": 0, "neu": 1, "neg": 2, "q": 3}
* Size: 26,737 messages
* Language: Central Thai
* Style: Informal and conversational. With some news headlines and advertisement.
* Time period: Around 2016 to early 2019. With small amount from other period.
* Domains: Mixed. Majority are consumer products and services (restaurants, cosmetics, drinks, car, hotels), with some current affairs.
* Privacy:
    * Only messages that made available to the public on the internet (websites, blogs, social network sites).
    * For Facebook, this means the public comments (everyone can see) that made on a public page.
    * Private/protected messages and messages in groups, chat, and inbox are not included.
* Alternations and modifications:
    * Keep in mind that this corpus does not statistically represent anything in the language register.
    * Large amount of messages are not in their original form. Personal data are removed or masked.
    * Duplicated, leading, and trailing whitespaces are removed. Other punctuations, symbols, and emojis are kept intact.
    (Mis)spellings are kept intact.
    * Messages longer than 2,000 characters are removed.
    * Long non-Thai messages are removed. Duplicated message (exact match) are removed.
* More characteristics of the data can be explore: https://github.com/PyThaiNLP/wisesight-sentiment/blob/master/exploration.ipynb
"""


class WisesightSentimentConfig(datasets.BuilderConfig):
    """BuilderConfig for WisesightSentiment."""

    def __init__(self, **kwargs):
        """BuilderConfig for WisesightSentiment.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(WisesightSentimentConfig, self).__init__(**kwargs)


class WisesightSentiment(datasets.GeneratorBasedBuilder):
    """Wisesight Sentiment Corpus: Social media messages in Thai language with sentiment category (positive, neutral, negative, question)"""

    _DOWNLOAD_URL = "https://github.com/PyThaiNLP/wisesight-sentiment/raw/master/huggingface/data.zip"
    _TRAIN_FILE = "train.jsonl"
    _VAL_FILE = "valid.jsonl"
    _TEST_FILE = "test.jsonl"

    BUILDER_CONFIGS = [
        WisesightSentimentConfig(
            name="wisesight_sentiment",
            version=datasets.Version("1.0.0"),
            description="Wisesight Sentiment Corpus: Social media messages in Thai language with sentiment category (positive, neutral, negative, question)",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "texts": datasets.Value("string"),
                    "category": datasets.features.ClassLabel(names=["pos", "neu", "neg", "q"]),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/PyThaiNLP/wisesight-sentiment",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="texts", label_column="category")],
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FILE)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, self._VAL_FILE)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FILE)},
            ),
        ]

    def _generate_examples(self, filepath):
        """Generate WisesightSentiment examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                texts = data["texts"]
                category = data["category"]
                yield id_, {"texts": texts, "category": category}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""CodeSearchNet corpus: proxy dataset for semantic code search"""

# TODO: add licensing info in the examples
# TODO: log richer informations (especially while extracting the jsonl.gz files)
# TODO: enable custom configs; such as: "java+python"
# TODO: enable fetching examples with a given license, eg: "java_MIT"


import json
import os

import datasets


_CITATION = """\
@article{husain2019codesearchnet,
    title={{CodeSearchNet} challenge: Evaluating the state of semantic code search},
    author={Husain, Hamel and Wu, Ho-Hsiang and Gazit, Tiferet and Allamanis, Miltiadis and Brockschmidt, Marc},
    journal={arXiv preprint arXiv:1909.09436},
    year={2019}
}
"""

_DESCRIPTION = """\
CodeSearchNet corpus contains about 6 million functions from open-source code \
spanning six programming languages (Go, Java, JavaScript, PHP, Python, and Ruby). \
The CodeSearchNet Corpus also contains automatically generated query-like \
natural language for 2 million functions, obtained from mechanically scraping \
and preprocessing associated function documentation.
"""

_HOMEPAGE = "https://github.com/github/CodeSearchNet"

_LICENSE = "Various"

_S3_BUCKET_URL = "https://s3.amazonaws.com/code-search-net/CodeSearchNet/v2/"
_AVAILABLE_LANGUAGES = ["python", "java", "javascript", "go", "ruby", "php"]
_URLs = {language: _S3_BUCKET_URL + f"{language}.zip" for language in _AVAILABLE_LANGUAGES}
# URLs for "all" are just the concatenation of URLs for all languages
_URLs["all"] = _URLs.copy()


class CodeSearchNet(datasets.GeneratorBasedBuilder):
    """ "CodeSearchNet corpus: proxy dataset for semantic code search."""

    VERSION = datasets.Version("1.0.0", "Add CodeSearchNet corpus dataset")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="all",
            version=VERSION,
            description="All available languages: Java, Go, Javascript, Python, PHP, Ruby",
        ),
        datasets.BuilderConfig(
            name="java",
            version=VERSION,
            description="Java language",
        ),
        datasets.BuilderConfig(
            name="go",
            version=VERSION,
            description="Go language",
        ),
        datasets.BuilderConfig(
            name="python",
            version=VERSION,
            description="Pyhton language",
        ),
        datasets.BuilderConfig(
            name="javascript",
            version=VERSION,
            description="Javascript language",
        ),
        datasets.BuilderConfig(
            name="ruby",
            version=VERSION,
            description="Ruby language",
        ),
        datasets.BuilderConfig(
            name="php",
            version=VERSION,
            description="PHP language",
        ),
    ]

    DEFAULT_CONFIG_NAME = "all"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "repository_name": datasets.Value("string"),
                    "func_path_in_repository": datasets.Value("string"),
                    "func_name": datasets.Value("string"),
                    "whole_func_string": datasets.Value("string"),
                    "language": datasets.Value("string"),
                    "func_code_string": datasets.Value("string"),
                    "func_code_tokens": datasets.Sequence(datasets.Value("string")),
                    "func_documentation_string": datasets.Value("string"),
                    "func_documentation_tokens": datasets.Sequence(datasets.Value("string")),
                    "split_name": datasets.Value("string"),
                    "func_code_url": datasets.Value("string"),
                    # TODO - add licensing info in the examples
                }
            ),
            # No default supervised keys
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators.

        Note: The original data is stored in S3, and follows this unusual directory structure:
            ```
            .
            ├── <language_name>  # e.g. python
            │   └── final
            │       └── jsonl
            │           ├── test
            │           │   └── <language_name>_test_0.jsonl.gz
            │           ├── train
            │           │   ├── <language_name>_train_0.jsonl.gz
            │           │   ├── <language_name>_train_1.jsonl.gz
            │           │   ├── ...
            │           │   └── <language_name>_train_n.jsonl.gz
            │           └── valid
            │               └── <language_name>_valid_0.jsonl.gz
            ├── <language_name>_dedupe_definitions_v2.pkl
            └── <language_name>_licenses.pkl
            ```
        """
        data_urls = _URLs[self.config.name]
        if isinstance(data_urls, str):
            data_urls = {self.config.name: data_urls}
        # Download & extract the language archives
        data_dirs = [
            os.path.join(directory, lang, "final", "jsonl")
            for lang, directory in dl_manager.download_and_extract(data_urls).items()
        ]

        split2dirs = {
            split_name: [os.path.join(directory, split_name) for directory in data_dirs]
            for split_name in ["train", "test", "valid"]
        }

        split2paths = dl_manager.extract(
            {
                split_name: [
                    os.path.join(directory, entry_name)
                    for directory in split_dirs
                    for entry_name in os.listdir(directory)
                ]
                for split_name, split_dirs in split2dirs.items()
            }
        )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": split2paths["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepaths": split2paths["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepaths": split2paths["valid"],
                },
            ),
        ]

    def _generate_examples(self, filepaths):
        """Yields the examples by iterating through the available jsonl files."""
        for file_id_, filepath in enumerate(filepaths):
            with open(filepath, encoding="utf-8") as f:
                for row_id_, row in enumerate(f):
                    # Key of the example = file_id + row_id,
                    # to ensure all examples have a distinct key
                    id_ = f"{file_id_}_{row_id_}"
                    data = json.loads(row)
                    yield id_, {
                        "repository_name": data["repo"],
                        "func_path_in_repository": data["path"],
                        "func_name": data["func_name"],
                        "whole_func_string": data["original_string"],
                        "language": data["language"],
                        "func_code_string": data["code"],
                        "func_code_tokens": data["code_tokens"],
                        "func_documentation_string": data["docstring"],
                        "func_documentation_tokens": data["docstring_tokens"],
                        "split_name": data["partition"],
                        "func_code_url": data["url"],
                    }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import datasets


_DESCRIPTION = """\
Webbnyheter 2012 from Spraakbanken, semi-manually annotated and adapted for CoreNLP Swedish NER. Semi-manually defined in this case as: Bootstrapped from Swedish Gazetters then manually correcte/reviewed by two independent native speaking swedish annotators. No annotator agreement calculated.
"""
_HOMEPAGE_URL = "https://github.com/klintan/swedish-ner-corpus"
_TRAIN_URL = "https://raw.githubusercontent.com/klintan/swedish-ner-corpus/master/train_corpus.txt"
_TEST_URL = "https://raw.githubusercontent.com/klintan/swedish-ner-corpus/master/test_corpus.txt"
_CITATION = None


class SwedishNERCorpus(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(names=["0", "LOC", "MISC", "ORG", "PER"])
                    ),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_URL)
        test_path = dl_manager.download_and_extract(_TEST_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": train_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"datapath": test_path},
            ),
        ]

    def _generate_examples(self, datapath):
        sentence_counter = 0
        with open(datapath, encoding="utf-8") as f:
            current_words = []
            current_labels = []
            for row in f:
                row = row.rstrip()
                row_split = row.split()
                if len(row_split) == 2:
                    token, label = row_split
                    current_words.append(token)
                    current_labels.append(label)
                else:
                    if not current_words:
                        continue
                    assert len(current_words) == len(current_labels), "word len doesnt match label length"
                    sentence = (
                        sentence_counter,
                        {
                            "id": str(sentence_counter),
                            "tokens": current_words,
                            "ner_tags": current_labels,
                        },
                    )
                    sentence_counter += 1
                    current_words = []
                    current_labels = []
                    yield sentence

            # if something remains:
            if current_words:
                sentence = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "tokens": current_words,
                        "ner_tags": current_labels,
                    },
                )
                yield sentence



import csv
import os

import datasets


_CITATION = """\
@article{go2009twitter,
  title={Twitter sentiment classification using distant supervision},
  author={Go, Alec and Bhayani, Richa and Huang, Lei},
  journal={CS224N project report, Stanford},
  volume={1},
  number={12},
  pages={2009},
  year={2009}
}
"""

_DESCRIPTION = """\
Sentiment140 consists of Twitter messages with emoticons, which are used as noisy labels for
sentiment classification. For more detailed information please refer to the paper.
"""
_URL = "http://help.sentiment140.com/home"
_DATA_URL = "http://cs.stanford.edu/people/alecmgo/trainingandtestdata.zip"

_TEST_FILE_NAME = "testdata.manual.2009.06.14.csv"
_TRAIN_FILE_NAME = "training.1600000.processed.noemoticon.csv"


class Sentiment140Config(datasets.BuilderConfig):

    """BuilderConfig for Break"""

    def __init__(self, data_url, **kwargs):
        """BuilderConfig for BlogAuthorship

        Args:
          data_url: `string`, url to the dataset (word or raw level)
          **kwargs: keyword arguments forwarded to super.
        """
        super(Sentiment140Config, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.data_url = data_url


class Sentiment140(datasets.GeneratorBasedBuilder):

    VERSION = datasets.Version("0.1.0")
    BUILDER_CONFIGS = [
        Sentiment140Config(
            name="sentiment140",
            data_url=_DATA_URL,
            description="sentiment classification dataset. Twitter messages are classified as either 'positive'=0, 'neutral'=1 or 'negative'=2.",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "user": datasets.Value("string"),
                    "sentiment": datasets.Value("int32"),
                    "query": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_DATA_URL)

        test_csv_file = os.path.join(data_dir, _TEST_FILE_NAME)
        train_csv_file = os.path.join(data_dir, _TRAIN_FILE_NAME)

        if self.config.name == "sentiment140":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"file_path": train_csv_file},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"file_path": test_csv_file},
                ),
            ]
        else:
            raise NotImplementedError(f"{self.config.name} does not exist")

    def _generate_examples(self, file_path):
        """Yields examples."""

        with open(file_path, encoding="ISO-8859-1") as f:
            data = csv.reader(f, delimiter=",", quotechar='"')
            for row_id, row in enumerate(data):
                sentiment, tweet_id, date, query, user_name, message = row
                yield f"{row_id}_{tweet_id}", {
                    "text": message,
                    "date": date,
                    "user": user_name,
                    "sentiment": int(sentiment),
                    "query": query,
                }



"""HebrewSentiment: A Modern Hebrew Sentiment Analysis Dataset."""


import csv

import datasets


_DESCRIPTION = """\
HebrewSentiment is a data set consists of 12,804 user comments to posts on the official Facebook page of Israel’s
president, Mr. Reuven Rivlin. In October 2015, we used the open software application Netvizz (Rieder,
2013) to scrape all the comments to all of the president’s posts in the period of June – August 2014,
the first three months of Rivlin’s presidency.2 While the president’s posts aimed at reconciling tensions
and called for tolerance and empathy, the sentiment expressed in the comments to the president’s posts
was polarized between citizens who warmly thanked the president, and citizens that fiercely critiqued his
policy. Of the 12,804 comments, 370 are neutral; 8,512 are positive, 3,922 negative.

Data Annotation: A trained researcher examined each comment and determined its sentiment value,
where comments with an overall positive sentiment were assigned the value 1, comments with an overall
negative sentiment were assigned the value -1, and comments that are off-topic to the post’s content
were assigned the value 0. We validated the coding scheme by asking a second trained researcher to
code the same data. There was substantial agreement between raters (N of agreements: 10623, N of
disagreements: 2105, Coehn’s Kappa = 0.697, p = 0).
"""

_CITATION = """\
@inproceedings{amram-etal-2018-representations,
    title = "Representations and Architectures in Neural Sentiment Analysis for Morphologically Rich Languages: A Case Study from {M}odern {H}ebrew",
    author = "Amram, Adam  and
      Ben David, Anat  and
      Tsarfaty, Reut",
    booktitle = "Proceedings of the 27th International Conference on Computational Linguistics",
    month = aug,
    year = "2018",
    address = "Santa Fe, New Mexico, USA",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/C18-1190",
    pages = "2242--2252",
    abstract = "This paper empirically studies the effects of representation choices on neural sentiment analysis for Modern Hebrew, a morphologically rich language (MRL) for which no sentiment analyzer currently exists. We study two dimensions of representational choices: (i) the granularity of the input signal (token-based vs. morpheme-based), and (ii) the level of encoding of vocabulary items (string-based vs. character-based). We hypothesise that for MRLs, languages where multiple meaning-bearing elements may be carried by a single space-delimited token, these choices will have measurable effects on task perfromance, and that these effects may vary for different architectural designs {---} fully-connected, convolutional or recurrent. Specifically, we hypothesize that morpheme-based representations will have advantages in terms of their generalization capacity and task accuracy, due to their better OOV coverage. To empirically study these effects, we develop a new sentiment analysis benchmark for Hebrew, based on 12K social media comments, and provide two instances of these data: in token-based and morpheme-based settings. Our experiments show that representation choices empirical effects vary with architecture type. While fully-connected and convolutional networks slightly prefer token-based settings, RNNs benefit from a morpheme-based representation, in accord with the hypothesis that explicit morphological information may help generalize. Our endeavour also delivers the first state-of-the-art broad-coverage sentiment analyzer for Hebrew, with over 89% accuracy, alongside an established benchmark to further study the effects of linguistic representation choices on neural networks{'} task performance.",
}
"""

_TRAIN_TOKEN_DOWNLOAD_URL = (
    "https://github.com/omilab/Neural-Sentiment-Analyzer-for-Modern-Hebrew/blob/master/data/token_train.tsv?raw=true"
)
_TEST_TOKEN_DOWNLOAD_URL = (
    "https://github.com/omilab/Neural-Sentiment-Analyzer-for-Modern-Hebrew/blob/master/data/token_test.tsv?raw=true"
)
_TRAIN_MORPH_DOWNLOAD_URL = (
    "https://github.com/omilab/Neural-Sentiment-Analyzer-for-Modern-Hebrew/blob/master/data/morph_train.tsv?raw=true"
)
_TEST_MORPH_DOWNLOAD_URL = (
    "https://github.com/omilab/Neural-Sentiment-Analyzer-for-Modern-Hebrew/blob/master/data/morph_test.tsv?raw=true"
)


class HebrewSentimentConfig(datasets.BuilderConfig):
    """BuilderConfig for HebrewSentiment."""

    def __init__(self, granularity="token", **kwargs):
        """BuilderConfig for HebrewSentiment.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        assert granularity in {"token", "morph"}
        self.granularity = granularity
        super(HebrewSentimentConfig, self).__init__(**kwargs)


class HebrewSentiment(datasets.GeneratorBasedBuilder):
    """HebrewSentiment: A Modern Hebrew Sentiment Analysis Dataset."""

    BUILDER_CONFIG_CLASS = HebrewSentimentConfig

    DEFAULT_CONFIG_NAME = "token"

    BUILDER_CONFIGS = [
        HebrewSentimentConfig(
            name="token",
            version=datasets.Version("1.0.0", ""),
            description="Hebrew Sentiment Dataset with token-based tokenization.",
            granularity="token",
        ),
        HebrewSentimentConfig(
            name="morph",
            version=datasets.Version("1.0.0", ""),
            description="Hebrew Sentiment Dataset with morpheme-based tokenization.",
            granularity="morph",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["pos", "neg", "off-topic"]),
                }
            ),
            homepage="https://github.com/omilab/Neural-Sentiment-Analyzer-for-Modern-Hebrew",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_url = _TRAIN_TOKEN_DOWNLOAD_URL if self.config.granularity == "token" else _TRAIN_MORPH_DOWNLOAD_URL
        train_path = dl_manager.download_and_extract(train_url)
        test_url = _TEST_TOKEN_DOWNLOAD_URL if self.config.granularity == "token" else _TEST_MORPH_DOWNLOAD_URL
        test_path = dl_manager.download_and_extract(test_url)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Hebrew Sentiment examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.DictReader(csv_file, fieldnames=["text", "label"], delimiter="\t")
            for id_, row in enumerate(csv_reader):
                row["label"] = int(row["label"])
                yield id_, row



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ArmanPerosNERCorpus - the first manually-annotated Persian NER corpus."""


import os

import datasets


_CITATION = """\
@inproceedings{poostchi-etal-2016-personer,
    title = "{P}erso{NER}: {P}ersian Named-Entity Recognition",
    author = "Poostchi, Hanieh  and
      Zare Borzeshi, Ehsan  and
      Abdous, Mohammad  and
      Piccardi, Massimo",
    booktitle = "Proceedings of {COLING} 2016, the 26th International Conference on Computational Linguistics: Technical Papers",
    month = dec,
    year = "2016",
    address = "Osaka, Japan",
    publisher = "The COLING 2016 Organizing Committee",
    url = "https://www.aclweb.org/anthology/C16-1319",
    pages = "3381--3389",
    abstract = "Named-Entity Recognition (NER) is still a challenging task for languages with low digital resources. The main difficulties arise from the scarcity of annotated corpora and the consequent problematic training of an effective NER pipeline. To abridge this gap, in this paper we target the Persian language that is spoken by a population of over a hundred million people world-wide. We first present and provide ArmanPerosNERCorpus, the first manually-annotated Persian NER corpus. Then, we introduce PersoNER, an NER pipeline for Persian that leverages a word embedding and a sequential max-margin classifier. The experimental results show that the proposed approach is capable of achieving interesting MUC7 and CoNNL scores while outperforming two alternatives based on a CRF and a recurrent neural network.",
}
"""

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
The dataset includes 250,015 tokens and 7,682 Persian sentences in total. It is available in 3 folds to be used in turn as training and test sets. The NER tags are in IOB format.
"""

_HOMEPAGE = ""

_LICENSE = "Creative Commons Attribution 4.0 International License"

_URL = "https://github.com/HaniehP/PersianNER/raw/master/ArmanPersoNERCorpus.zip"


class PersianNER(datasets.GeneratorBasedBuilder):
    """ArmanPerosNERCorpus - the first manually-annotated Persian NER corpus."""

    VERSION = datasets.Version("1.1.0")

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.

    # If you need to make complex sub-parts in the datasets with configurable options
    # You can create your own builder configuration class to store attribute, inheriting from datasets.BuilderConfig
    # BUILDER_CONFIG_CLASS = MyBuilderConfig

    # You will be able to load one or the other configurations in the following list with
    # data = datasets.load_dataset('my_dataset', 'first_domain')
    # data = datasets.load_dataset('my_dataset', 'second_domain')
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="fold1", version=VERSION, description="This is the first fold  of the ArmanPersoNERCorpus"
        ),
        datasets.BuilderConfig(
            name="fold2", version=VERSION, description="This is the second fold  of the ArmanPersoNERCorpus"
        ),
        datasets.BuilderConfig(
            name="fold3", version=VERSION, description="This is the third fold  of the ArmanPersoNERCorpus"
        ),
    ]

    DEFAULT_CONFIG_NAME = "fold1"  # It's not mandatory to have a default configuration. Just use one if it make sense.

    def _info(self):
        features = datasets.Features(
            {
                "tokens": datasets.Sequence(datasets.Value("string")),
                "ner_tags": datasets.Sequence(
                    datasets.ClassLabel(
                        names=[
                            "O",
                            "I-event",
                            "I-fac",
                            "I-loc",
                            "I-org",
                            "I-pers",
                            "I-pro",
                            "B-event",
                            "B-fac",
                            "B-loc",
                            "B-org",
                            "B-pers",
                            "B-pro",
                        ]
                    )
                ),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=("tokens", "ner_tags"),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URL
        data_dir = dl_manager.download_and_extract(my_urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, f"train_{self.config.name}.txt"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, f"test_{self.config.name}.txt"), "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, "r", encoding="utf-8") as f:
            id_ = 0
            tokens = []
            ner_labels = []
            for line in f:
                stripped_line = line.strip(" \n")  # strip away whitespaces AND new line characters
                if len(stripped_line) == 0:
                    # If line is empty, it means we reached the end of a sentence.
                    # We can yield the tokens and labels
                    if len(tokens) > 0 and len(ner_labels) > 0:
                        yield id_, {
                            "tokens": tokens,
                            "ner_tags": ner_labels,
                        }
                    else:
                        # Do not yield if tokens or ner_labels is empty
                        # It can be the case if several empty lines are contiguous
                        continue
                    # Then we need to increment the _id and reset the tokens and ner_labels list
                    id_ += 1
                    tokens = []
                    ner_labels = []
                else:
                    token, ner_label = line.split(" ")  # Retrieve token and label
                    tokens.append(token)
                    ner_labels.append(ner_label)
